Type: Package
Package: OmopStudyBuilder
Title: Build Reproducible Network Studies for OMOP Common Data Model
Version: 0.1.1
Authors@R: c(
    person("Folu", "Akintola", , "folu.akintola@ndorms.ox.ac.uk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0588-2298")),
    person("Edward", "Burn", , "edward.burn@ndorms.ox.ac.uk", role = "aut",
           comment = c(ORCID = "0000-0002-9286-1128")),
    person("Martí", "Català", , "marti.catalasabate@ndorms.ox.ac.uk", role = "aut",
           comment = c(ORCID = "0000-0003-3308-9905")),
    person("Marta", "Alcalde-Herraiz", , "marta.alcaldeherraiz@ndorms.ox.ac.uk", role = "ctb",
           comment = c(ORCID = "0009-0002-4405-1814"))
  )
Description: Streamlines the setup and execution of network studies using
    the Observational Medical Outcomes Partnership (OMOP) Common Data
    Model (CDM). Creates standardised project structures with template
    code, manages dependencies with 'renv', provides code review
    utilities, and supports containerised execution with 'Docker' for
    reproducible multi-site studies.  Includes 'GitHub' integration for
    collaboration and version control.
License: Apache License (>= 2)
URL: https://oxford-pharmacoepi.github.io/OmopStudyBuilder/
Depends: R (>= 4.1.0)
Imports: cli, jsonlite, omopgenerics, rlang, whisker
Suggests: dplyr, gert, getPass, gh, here, knitr, processx, purrr, renv,
        rmarkdown, scales, stringr, testthat
VignetteBuilder: knitr
Encoding: UTF-8
BugReports: https://github.com/oxford-pharmacoepi/OmopStudyBuilder/issues
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-11 21:45:18 UTC; foluwa
Author: Folu Akintola [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0588-2298>),
  Edward Burn [aut] (ORCID: <https://orcid.org/0000-0002-9286-1128>),
  Martí Català [aut] (ORCID: <https://orcid.org/0000-0003-3308-9905>),
  Marta Alcalde-Herraiz [ctb] (ORCID:
    <https://orcid.org/0009-0002-4405-1814>)
Maintainer: Folu Akintola <folu.akintola@ndorms.ox.ac.uk>
Repository: CRAN
Date/Publication: 2026-05-11 22:10:12 UTC
Built: R 4.5.2; ; 2026-05-12 01:00:12 UTC; unix

# Generated by roxygen2: do not edit by hand

export(dockeriseStudy)
export(initStudy)
export(linkGitHub)
export(pushDockerImage)
export(reviewStudy)
export(runRStudio)
export(runStudy)
export(stopStudy)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,read_json)
importFrom(rlang,.data)
importFrom(rlang,.env)
importFrom(utils,browseURL)
importFrom(utils,head)
importFrom(utils,install.packages)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# OmopStudyBuilder <img src="man/figures/image.jfif" align="right" style="height:100px; background: var(--bs-body-bg, #fff); padding-left: 12px; position: relative; z-index: 1;" alt="OmopStudyBuilder logo" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/oxford-pharmacoepi/OmopStudyBuilder/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/oxford-pharmacoepi/OmopStudyBuilder/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/OmopStudyBuilder)](https://CRAN.R-project.org/package=OmopStudyBuilder)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![Codecov test
coverage](https://codecov.io/gh/oxford-pharmacoepi/OmopStudyBuilder/branch/main/graph/badge.svg)](https://app.codecov.io/gh/oxford-pharmacoepi/OmopStudyBuilder?branch=main)

<!-- badges: end -->

The **OmopStudyBuilder** package helps a study prepare for network
studies using the OMOP Common Data Model (CDM). The package sets up an R
project for your study with a default folder structure and template code
typically required for network studies. This allows you to focus on the
parts of the analysis that are specific to your study design.

In addition to project setup, the package reviews code and dependencies,
supports renv locking for consistent package versions, and can build and
run a Docker image for reproducible execution aligned with best
practices.

The package is highly opinionated and designed to align with the OxInfer
study code checklist. For further details, please refer to the
[documentation](https://oxford-pharmacoepi.github.io/Oxinfer/onboarding/code_review.html).

## Installation

You can install the development version of the package from GitHub:

``` r
# install.packages("remotes")
remotes::install_github("oxford-pharmacoepi/OmopStudyBuilder")
```

## Quick Start

The main entry point is `initStudy()`, which creates the study folder
structure and template files.

``` r
library(OmopStudyBuilder)

initStudy(here::here("SampleStudy"))
```

Once the study has been created, you can review the generated code and
dependencies:

``` r
reviewStudyCode(here::here("SampleStudy", "studyCode"))
reviewStudyDependencies(here::here("SampleStudy", "studyCode"))
```

If you want reproducible package versions, initialise `renv` in the
study folder and snapshot the environment:

``` r
renv::init(here::here("SampleStudy", "studyCode"))
install.packages(c("dplyr", "CDMConnector", "IncidencePrevalence"))
renv::snapshot(here::here("SampleStudy", "studyCode"))
```

## Optional: Docker for reproducible execution

This package supports building and running study code in Docker for
reproducible execution. Docker is optional and only needed if you want
to build and run the study in a containerised workflow. Install Docker
and confirm it is running before using the Docker-based functions.

**General checks (all operating systems)**

- Install Docker.
- Start Docker (e.g., Docker Desktop).
- Verify Docker is available:
  - `docker --version`
  - `docker info`

**macOS**

- Install Docker Desktop for Mac:
  <https://docs.docker.com/desktop/setup/install/mac-install/>
- Open Docker Desktop and wait until it reports “Docker Desktop is
  running”.

**Windows**

- Install Docker Desktop for Windows (WSL 2 recommended):
  <https://docs.docker.com/desktop/setup/install/windows-install/>
- Ensure WSL 2 is enabled (Docker Desktop installer will guide you).
- Open Docker Desktop and wait until it is running.

**Linux**

- Install Docker Engine using your distribution instructions:
  <https://docs.docker.com/engine/install/>
- Start/enable the Docker service (varies by distro).
- If you want to run Docker without `sudo`, follow:
  <https://docs.docker.com/engine/install/linux-postinstall/>

### Example Usage

After the study has been created and configured, you can optionally
build a Docker image from the study folder:

``` r
dockeriseStudy(path = here::here("SampleStudy", "studyCode"))
```

Run the study interactively in RStudio Server or as an automated script.
If you use a `.env` file for credentials, pass `env_file = ".env"`.

``` r
runRStudio()
runStudy()
```

Use optional inputs only if you need them:

``` r
runStudy(
  image_name = "omop-study-study-code",
  data_path = "path/to/data",
  results_path = "./results"
)
```

To distribute the study, share the **study folder** created by
`initStudy()` (including `studyCode/` and `renv.lock`). Partners can
build and run using the same commands.

``` r
install.packages("OmopStudyBuilder")
library(OmopStudyBuilder)

dockeriseStudy(path = here::here("SampleStudy", "studyCode"))
runStudy()
runRStudio()
stopStudy()
```

To push a built image to Docker Hub, use the helper and enter your
credentials when prompted (the tag defaults to `latest`, and the image
name defaults to the current folder name):

``` r
pushDockerImage(
  repo = "yourname/omop-study-study-code"
)
```

# Study Code

This folder contains the main study analysis code.

## Instructions

For detailed instructions on how to run the study code, please see the [INSTRUCTIONS.md](../INSTRUCTIONS.md) file in the root directory of this repository.

# {{REPO_NAME}}

This is an OMOP CDM study repository.

Created: {{DATE}}

## Repository Structure

- `studyCode/` - Analysis code and cohort definitions
- `studyShiny/` - Interactive results viewer
- `diagnosticsCode/` - Cohort diagnostics code
- `diagnosticsShiny/` - Diagnostics results viewer
- `results/` - Study results (excluded from version control)

## About

This repository was created with [OmopStudyBuilder](https://github.com/oxford-pharmacoepi/OmopStudyBuilder), an R package for building reproducible OMOP CDM network studies.

# Diagnostics Shiny App

This folder is a scaffold for a Shiny app used to explore diagnostic outputs.

## Instructions

For detailed instructions on how to run the diagnostics shiny app, please see the [INSTRUCTIONS.md](../INSTRUCTIONS.md) file in the root directory of this repository.

# Diagnostics Code

This folder contains the diagnostic code needed before running the main study analysis.

## Instructions

For detailed instructions on how to run the diagnostics code, please see the [INSTRUCTIONS.md](../INSTRUCTIONS.md) file in the root directory of this repository.

<!-- README.md is generated from README.Rmd. Please edit that file -->

# OmopStudyBuilder <a href="https://github.com/oxford-pharmacoepi/OmopStudyBuilder"><img src="man/figures/image.jfif" align="right" height="138" alt="OmopStudyBuilder website" /></a>

<!-- badges: start -->

[![R-CMD-check](https://github.com/oxford-pharmacoepi/OmopStudyBuilder/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/oxford-pharmacoepi/OmopStudyBuilder/actions/workflows/R-CMD-check.yaml)
[![CRAN
status](https://www.r-pkg.org/badges/version/OmopStudyBuilder)](https://CRAN.R-project.org/package=OmopStudyBuilder)
[![Lifecycle:
experimental](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![Codecov test
coverage](https://codecov.io/gh/oxford-pharmacoepi/OmopStudyBuilder/branch/main/graph/badge.svg)](https://app.codecov.io/gh/oxford-pharmacoepi/OmopStudyBuilder?branch=main)
<!-- badges: end -->

The goal of OmopStudyBuilder is to help you prepare study for network
studies using the OMOP CDM. The package takes care of setting up a R
project for your study with the default structure and generic code that
would be used for most network studies. This then leaves you to add the
code that is unique to your study.

The package is highly opinionated and designed to align with the OxInfer
study code checklist
<https://oxford-pharmacoepi.github.io/Oxinfer/onboarding/code_review.html>

# renv::activate()
# renv::restore()

library(CDMConnector)
library(DBI)
library(dplyr)
library(here)
library(OmopSketch)
library(omopgenerics)
library(CohortConstructor)
library(PhenotypeR)
library(CohortCharacteristics)
library(PatientProfiles)
library(stringr)
library(CodelistGenerator)
library(odbc)
library(RPostgres)
library(readr)
library(purrr)

# database metadata and connection details
# The name/ acronym for the database
db_name <- ""

# Database connection details
# In this study we also use the DBI package to connect to the database
# set up the dbConnect details below
# https://darwin-eu.github.io/CDMConnector/articles/DBI_connection_examples.html
# for more details.
# you may need to install another package for this
# eg for postgres
# db <- dbConnect(
#   RPostgres::Postgres(),
#   dbname = server_dbi,i
#   port = port,
#   host = host,
#   user = user,
#   password = password
# )
db <- dbConnect()

# The name of the schema that contains the OMOP CDM with patient-level data
cdm_schema <- ""

# A prefix for all permanent tables in the database
write_prefix <- ""

# The name of the schema where results tables will be created
write_schema <- ""

# The name of the schema where the achilles tables are
achilles_schema <- ""

# minimum counts that can be displayed according to data governance
minCellCount <- 5

# Create cdm object ----
cdm <- cdmFromCon(
  con = db,
  cdmSchema = cdm_schema,
  writeSchema = write_schema,
  writePrefix = write_prefix,
  cdmName = db_name,
  achillesSchema = achilles_schema
)

# Run study ----
source(here("run_study.R"))

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(OmopStudyBuilder)

## ----dockeriseStudy-example, eval=FALSE---------------------------------------
# # From the studyCode folder (or pass path=...)
# dockeriseStudy(
#   image_name = "my-study",
#   path = "./studyCode",
#   useRStudio = FALSE
# )

## ----dockeriseStudy-rstudio, eval=FALSE---------------------------------------
# dockeriseStudy(
#   image_name = "my-study",
#   path = "./studyCode",
#   useRStudio = TRUE
# )

## ----dockeriseStudy-github-token, eval=FALSE----------------------------------
# dockeriseStudy(
#   path = "./studyCode",
#   github_token = Sys.getenv("GITHUB_PAT")
# )

## ----runRStudio-example, eval=FALSE-------------------------------------------
# runRStudio(
#   image_name = "my-study",
#   results_path = "./results"
# )

## ----runStudy-example, eval=FALSE---------------------------------------------
# runStudy(
#   image_name = "my-study",
#   results_path = "./results",
#   data_path = "/path/to/cdm/data",
#   script_path = "codeToRun.R"
# )

## ----stopStudy-example, eval=FALSE--------------------------------------------
# stopStudy(image_name = "my-study")

## ----pushDockerImage-example, eval=FALSE--------------------------------------
# pushDockerImage(
#   image_name = "my-study",
#   repo = "yourusername/my-study"
# )

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, include=FALSE-----------------------------------------------------
library(OmopStudyBuilder)

## ----basic-usage, message=FALSE-----------------------------------------------
# Create a temporary root for this vignette
study_root <- file.path(tempdir(), "SampleStudy")

initStudy(study_root)

# Show top-level contents created by initStudy()
list.files(study_root)

## ----list-recursive-----------------------------------------------------------
list.files(study_root, recursive = TRUE)

## ----diagnostics-only, message=FALSE------------------------------------------
diag_root <- file.path(tempdir(), "DiagnosticsOnly")

initStudy(
  directory   = diag_root,
  diagnostics = TRUE,
  study       = FALSE
)

list.files(diag_root)

## ----study-only, message=FALSE------------------------------------------------
study_only_root <- file.path(tempdir(), "StudyOnly")

initStudy(
  directory   = study_only_root,
  diagnostics = FALSE,
  study       = TRUE
)

list.files(study_only_root)

## ----build-docker, eval=FALSE-------------------------------------------------
# # Build a Docker image containing your study code
# dockeriseStudy(
#   image_name = "omop-study-study-code",
#   path = file.path(study_root, "studyCode")
# )
# #> Building Docker image: omop-study-study-code
# #> This may take some minutes on first build...
# #> ✔ Image built successfully: omop-study-study-code

## ----push-image, eval=FALSE---------------------------------------------------
# pushDockerImage(
#   image_name = "omop-study-study-code",
#   repo = "yourusername/myomopstudy"
# )
# #> ✔ Image pushed to Docker Hub!

## ----pull-run, eval=FALSE-----------------------------------------------------
# library(OmopStudyBuilder)
# 
# # Interactive (requires image built with dockeriseStudy(useRStudio = TRUE)):
# runRStudio(
#   image_name = "yourusername/myomopstudy:latest",
#   results_path = "./results"
# )

## ----rstudio-run, eval=FALSE--------------------------------------------------
# # Requires image built with dockeriseStudy(useRStudio = TRUE)
# runRStudio(
#   image_name = "omop-study-study-code",
#   results_path = "./results"
# )

## ----auto-run, eval=FALSE-----------------------------------------------------
# runStudy(
#   image_name = "omop-study-study-code",
#   results_path = "./results",
#   data_path = "path/to/data"
# )

## ----stop-containers, eval=FALSE----------------------------------------------
# stopStudy(image_name = "omop-study-study-code")

# renv::activate()
# renv::restore()

library(DBI)
library(dplyr)
library(here)
library(CDMConnector)
library(omopgenerics)
library(OmopSketch)
library(CodelistGenerator)
library(CohortConstructor)
library(PatientProfiles)
library(CohortCharacteristics)
library(DrugUtilisation)
library(IncidencePrevalence)
library(CohortSurvival)
library(odbc)
library(RPostgres)

# database metadata and connection details
# The name/ acronym for the database
db_name <- "..."

# Database connection details
# db <- dbConnect(
#   RPostgres::Postgres(),
#   dbname = server_dbi,
#   port = port,
#   host = host,
#   user = user,
#   password = password
# )
db <- dbConnect()

# The name of the schema that contains the OMOP CDM with patient-level data
cdm_schema <- "..."

# A prefix for all permanent tables in the database
write_prefix <- "..."

# The name of the schema where results tables will be created
write_schema <- "..."

# minimum counts that can be displayed according to data governance
min_cell_count <- 5

# Create cdm object ----
cdm <- cdmFromCon(
  con = db,
  cdmSchema = cdm_schema,
  writeSchema = write_schema,
  writePrefix = write_prefix,
  cdmName = db_name
)

# Run the study
source(here("runStudy.R"))

# Check codeToRun inputs ----
omopgenerics::validateCdmArgument(cdm,
                                  requiredTables = c("person",
                                                     "observation_period",
                                                     "condition_occurrence",
                                                     "drug_exposure",
                                                     "concept"))
omopgenerics::assertNumeric(min_cell_count)

# Create a log file ----
createLogFile(logFile = tempfile(pattern = "log_{date}_{time}"))
logMessage("LOG CREATED")

# Define analysis settings -----
study_period <- c(as.Date(NA), as.Date(NA))

# Initialise list to store results as we go -----
results <- list()

# CDM modifications -----
# CDM summary -----
results[["snapshot"]] <- summariseOmopSnapshot(cdm)
results[["obs_period"]] <- summariseObservationPeriod(cdm$observation_period)

# Instantiate study cohorts ----
logMessage("Instantiating study cohorts")
source(here("cohorts", "instantiateCohorts.R"))
logMessage("Study cohorts instantiated")

# Cohort counts and attrition ----
# results[["counts"]] <- summariseCohortCount("...")
# results[["attrition"]] <- summariseCohortAttrition("...")

# Run analyses ----
logMessage("Run study analyses")
source(here("analyses", "cohortCharacteristics.R"))
source(here("analyses", "cohortSurvival.R"))
source(here("analyses", "drugUtilisation.R"))
source(here("analyses", "incidencePrevalence.R"))
logMessage("Analyses finished")

# Capture log file ----
results[["log"]] <- summariseLogFile(cdmName = omopgenerics::cdmName(cdm))

# Finish ----
results <- results |>
  vctrs::list_drop_empty() |>
  omopgenerics::bind()
exportSummarisedResult(results,
                       minCellCount = min_cell_count,
                       fileName = "results_{cdm_name}_{date}.csv",
                       path = here("results"))

cli::cli_alert_success("Study finished")

# create logger ----
resultsFolder <- here("results")
if(!dir.exists(resultsFolder)){
  dir.create(resultsFolder)
}

createLogFile(logFile = tempfile(pattern = "log_{date}_{time}"))
logMessage("LOG CREATED")

# run ----
source(here("cohorts", "instantiateCohorts.R"))
info(logger, "- Running PhenotypeDiagnostics")
diagnostics <- phenotypeDiagnostics(cdm$study_cohorts,
                          survival = FALSE,
                          cohortSample = 20000,
                          matchedSample = NULL,
                          populationSample = NULL)

exportSummarisedResult(diagnostics,
                       minCellCount = minCellCount,
                       fileName = "phenotyper_results_{cdm_name}_{date}.csv",
                       path = resultsFolder)
logMessage("Finished")

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom jsonlite fromJSON read_json
#' @importFrom rlang .data
#' @importFrom rlang .env
#' @importFrom utils head install.packages
## usethis namespace: end
NULL

#' Creates initial directory for an OMOP CDM network study
#'
#' @param directory Path to a directory that will be used as the root folder
#' for the study. If it does not exist, it will be created. The directory
#' must be empty if it already exists.
#'
#' @param diagnostics A single TRUE or FALSE value.
#'   If TRUE (the default), the function creates the `diagnosticsCode/`
#'   and `diagnosticsShiny/` folders using the package templates.
#'   If FALSE, these diagnostics folders are not created.
#'
#' @param study A single TRUE or FALSE value.
#'   If TRUE (the default), the function creates the `studyCode/`
#'   and `studyShiny/` folders using the package templates.
#'   If FALSE, these study folders are not created.
#'
#' @param studyTitle Character string with the study title. If NULL (default),
#'   uses the directory basename.
#'
#' @param studyLeads Character string with study leads. If NULL (default),
#'   leaves a placeholder.
#'
#' @param studyDescription Character string with study description. If NULL (default),
#'   leaves a placeholder.
#' 
#' @param repository Optional GitHub repository name. If provided, creates a GitHub
#'   repository and links it to the study. Requires the \code{gh} and \code{gert}
#'   packages and GitHub authentication (for example via \code{GITHUB_PAT}).
#'
#' @param organisation Optional GitHub organisation name. If NULL (default), creates
#'   repository under your personal account. Only used when \code{repository} is provided.
#'
#' @param private Logical. If TRUE (default), creates a private GitHub repository.
#'   Only used when \code{repository} is provided.
#'
#' @returns Project directory will be created
#' @export
#'
#' @examples
#' # Create a study called "SampleStudy" in a temporary directory
#' study_root <- file.path(tempdir(), "SampleStudy")
#' initStudy(study_root)
#'
#' # Inspect the top-level contents
#' list.files(study_root)
#'
#' # Create another study with custom metadata
#' study_root2 <- file.path(tempdir(), "DiabetesStudy")
#' initStudy(study_root2,
#'           studyTitle = "Diabetes Prevalence Study",
#'           studyLeads = "Dr. Smith, Dr. Jones")
#'
#' \dontrun{
#' # Create study with GitHub integration (requires GITHUB_PAT)
#' # Set PAT for current session:
#' Sys.setenv(GITHUB_PAT = "your_token_here")
#'
#' study_root3 <- file.path(tempdir(), "GitHubStudy")
#' initStudy(
#'   directory = study_root3,
#'   repository = "my-omop-study",
#'   organisation = "oxford-pharmacoepi",
#'   private = TRUE
#' )
#' }
initStudy <- function(directory,
                        diagnostics = TRUE,
                        study = TRUE,
                        studyTitle = NULL,
                        studyLeads = NULL,
                        studyDescription = NULL,
                        repository = NULL,
                        organisation = NULL,
                        private = TRUE) {
  validateRootDirectory(directory)
  omopgenerics::assertLogical(diagnostics, length = 1)
  omopgenerics::assertLogical(study, length = 1)
  omopgenerics::assertCharacter(studyTitle, length = 1, null = TRUE)
  omopgenerics::assertCharacter(studyLeads, length = 1, null = TRUE)
  omopgenerics::assertCharacter(studyDescription, length = 1, null = TRUE)
  omopgenerics::assertCharacter(repository, length = 1, null = TRUE)
  omopgenerics::assertCharacter(organisation, length = 1, null = TRUE)
  omopgenerics::assertLogical(private, length = 1)

  # Set smart defaults for template variables
  if (is.null(studyTitle)) {
    studyTitle <- basename(normalizePath(directory, mustWork = FALSE))
  }
  if (is.null(studyLeads)) {
    studyLeads <- "[Add study leads]"
  }
  if (is.null(studyDescription)) {
    studyDescription <- "[Add study description here]"
  }

  # Build folder structure description dynamically
  folderStructure <- c()
  if (isTRUE(diagnostics)) {
    folderStructure <- c(
      folderStructure,
        "- **[diagnosticsCode](diagnosticsCode/)**: Contains diagnostic code needed before running the main study",
        "- **[diagnosticsShiny](diagnosticsShiny/)**: Shiny app for exploring diagnostic outputs"
    )
  }
  if (isTRUE(study)) {
    folderStructure <- c(
      folderStructure,
        "- **[studyCode](studyCode/)**: Contains the main study analysis code",
        "- **[studyShiny](studyShiny/)**: Shiny app for exploring study results"
    )
  }
  folderStructure <- paste(folderStructure, collapse = "\n")

  # Prepare template data
  templateData <- list(
    STUDY_TITLE = studyTitle,
    STUDY_LEADS = studyLeads,
    STUDY_DESCRIPTION = studyDescription,
    STUDY_START = as.character(Sys.Date()),
    STUDY_END = "[Ongoing]",
    STUDY_STATUS = "Started",
    PUBLICATIONS = "[None yet]",
    FOLDER_STRUCTURE = folderStructure
  )

  # Process README template
  readmeTemplate <- readLines(system.file("study-README.md", package = "OmopStudyBuilder"))
  readmeContent <- whisker::whisker.render(readmeTemplate, templateData)
  writeLines(readmeContent, file.path(directory, "README.md"))

  # Process INSTRUCTIONS template
  instructionsTemplateData <- list(
    HAS_DIAGNOSTICS = isTRUE(diagnostics),
    HAS_STUDY = isTRUE(study)
  )
  instructionsTemplate <- readLines(system.file("INSTRUCTIONS.md", package = "OmopStudyBuilder"))
  instructionsContent <- whisker::whisker.render(instructionsTemplate, instructionsTemplateData)
  writeLines(instructionsContent, file.path(directory, "INSTRUCTIONS.md"))

  cli::cli_alert_success("{.strong {directory}} prepared as root folder for study.")

  if (isTRUE(diagnostics)) {
    directoryDiagnosticsCode <- file.path(directory, "diagnosticsCode")
    copyDirectory(
      from = system.file("diagnosticsCode", package = "OmopStudyBuilder"),
      to = directoryDiagnosticsCode
    )
    cli::cli_alert_success("{.strong {directoryDiagnosticsCode}} prepared for study diagnostics code")

    directoryDiagnosticsShiny <- file.path(directory, "diagnosticsShiny")
    copyDirectory(
      from = system.file("diagnosticsShiny", package = "OmopStudyBuilder"),
      to = directoryDiagnosticsShiny
    )
    cli::cli_alert_success("{.strong {directoryDiagnosticsShiny}} prepared for diagnostics shiny app")
  }

  if (isTRUE(study)) {
    directoryStudyCode <- file.path(directory, "studyCode")
    copyDirectory(
      from = system.file("studyCode", package = "OmopStudyBuilder"),
      to = directoryStudyCode
    )
    cli::cli_alert_success("{.strong {directoryStudyCode}} prepared for study study code")

    directoryStudyShiny <- file.path(directory, "studyShiny")
    copyDirectory(
      from = system.file("studyShiny", package = "OmopStudyBuilder"),
      to = directoryStudyShiny
    )
    cli::cli_alert_success("{.strong {directoryStudyShiny}} prepared for study shiny app")
  }

  # GitHub integration (optional)
  if (!is.null(repository)) {
    linkGitHub(
      directory = directory,
      repository = repository,
      organisation = organisation,
      private = private
    )
  }

  return(invisible())
}


copyDirectory <- function(from, to) {
  # files to copy
  oldFiles <- list.files(path = from, full.names = TRUE, recursive = TRUE)

  files <- list.files(path = from, full.names = FALSE, recursive = TRUE)
  NewFiles <- paste0(to, "/", files)

  dirsToCreate <- unique(dirname(NewFiles))
  sapply(dirsToCreate, dir.create, recursive = TRUE, showWarnings = FALSE)

  # copy files
  file.copy(from = oldFiles, to = NewFiles)
}

validateRootDirectory <- function(dir) {
  # root directory must be empty and create it if it does not exist
  if (!dir.exists(dir)) {
    dirCreated <- dir.create(dir, recursive = TRUE, showWarnings = FALSE)
    if (!dirCreated) {
      cli::cli_abort(c("Provided directory {.pkg {dir}} does not exist and could not be created."))
    }
  }

  if (length(list.files(dir, recursive = TRUE)) > 0) {
    cli::cli_abort(c("Provided directory {.pkg {dir}} is not empty."))
  }

  return(invisible())
}

test_that("directory set up", {

  # user can provide a directory that already exists (and is empty)
  temp_dir <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir, recursive = TRUE)
  expect_true(dir.exists(temp_dir))
  expect_no_error(initStudy(directory = temp_dir))
  expect_true("README.md" %in% list.files(temp_dir))

  # if directory does not exist, it should be created and used
  temp_dir <- here::here(tempdir(), omopgenerics::uniqueTableName())
  expect_false(dir.exists(temp_dir))
  expect_no_error(initStudy(directory = temp_dir))
  expect_true(dir.exists(temp_dir))
  expect_true("README.md" %in% list.files(temp_dir))

  # error if directory exists and contains files
  temp_dir_non_empty <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir_non_empty, recursive = TRUE)
  writeLines("some text", file.path(temp_dir_non_empty, "notes.txt"))
  expect_error(initStudy(directory = temp_dir_non_empty))

  # only diagnostics directories
  temp_dir_diag <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir_diag, recursive = TRUE)
  expect_no_error(initStudy(
    directory   = temp_dir_diag,
    diagnostics = TRUE,
    study       = FALSE
  ))
  expect_true("README.md" %in% list.files(temp_dir_diag))
  expect_true("diagnosticsCode" %in% list.files(temp_dir_diag))
  expect_true("diagnosticsShiny" %in% list.files(temp_dir_diag))
  expect_false("studyCode" %in% list.files(temp_dir_diag))
  expect_false("studyShiny" %in% list.files(temp_dir_diag))

  # only study directories
  temp_dir_study <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir_study, recursive = TRUE)
  expect_no_error(initStudy(
    directory   = temp_dir_study,
    diagnostics = FALSE,
    study       = TRUE
  ))
  expect_true("README.md" %in% list.files(temp_dir_study))
  expect_false("diagnosticsCode" %in% list.files(temp_dir_study))
  expect_false("diagnosticsShiny" %in% list.files(temp_dir_study))
  expect_true("studyCode" %in% list.files(temp_dir_study))
  expect_true("studyShiny" %in% list.files(temp_dir_study))
})

test_that("INSTRUCTIONS.md is created with correct content", {

  # INSTRUCTIONS.md should be created with both diagnostics and study
  temp_dir_both <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir_both, recursive = TRUE)
  expect_no_error(initStudy(
    directory   = temp_dir_both,
    diagnostics = TRUE,
    study       = TRUE
  ))
  expect_true("INSTRUCTIONS.md" %in% list.files(temp_dir_both))
  
  # Check content includes both diagnostics and study instructions
  instructions_both <- readLines(file.path(temp_dir_both, "INSTRUCTIONS.md"))
  instructions_text <- paste(instructions_both, collapse = " ")
  expect_true(grepl("Running the Diagnostics Code", instructions_text))
  expect_true(grepl("Running the Study Code", instructions_text))
  expect_true(grepl("diagnosticsCode\\.Rproj", instructions_text))
  expect_true(grepl("studyCode\\.Rproj", instructions_text))

  # INSTRUCTIONS.md with only diagnostics
  temp_dir_diag_only <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir_diag_only, recursive = TRUE)
  expect_no_error(initStudy(
    directory   = temp_dir_diag_only,
    diagnostics = TRUE,
    study       = FALSE
  ))
  expect_true("INSTRUCTIONS.md" %in% list.files(temp_dir_diag_only))
  
  instructions_diag <- readLines(file.path(temp_dir_diag_only, "INSTRUCTIONS.md"))
  instructions_diag_text <- paste(instructions_diag, collapse = " ")
  expect_true(grepl("Running the Diagnostics Code", instructions_diag_text))
  expect_false(grepl("Running the Study Code", instructions_diag_text))

  # INSTRUCTIONS.md with only study
  temp_dir_study_only <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir_study_only, recursive = TRUE)
  expect_no_error(initStudy(
    directory   = temp_dir_study_only,
    diagnostics = FALSE,
    study       = TRUE
  ))
  expect_true("INSTRUCTIONS.md" %in% list.files(temp_dir_study_only))
  
  instructions_study <- readLines(file.path(temp_dir_study_only, "INSTRUCTIONS.md"))
  instructions_study_text <- paste(instructions_study, collapse = " ")
  expect_false(grepl("Running the Diagnostics Code", instructions_study_text))
  expect_true(grepl("Running the Study Code", instructions_study_text))
})

test_that("README.md files point to INSTRUCTIONS.md", {
  temp_dir <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir, recursive = TRUE)
  initStudy(directory = temp_dir, diagnostics = TRUE, study = TRUE)
  
  # Check that subfolder READMEs reference the central instructions
  study_readme <- readLines(file.path(temp_dir, "studyCode", "README.md"))
  expect_true(any(grepl("\\.\\./INSTRUCTIONS\\.md", study_readme)))
})

test_that("README.md folder labels do not include trailing slashes", {
  temp_dir <- here::here(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir, recursive = TRUE)
  initStudy(directory = temp_dir, diagnostics = TRUE, study = TRUE)

  readme_lines <- readLines(file.path(temp_dir, "README.md"))
  readme_text <- paste(readme_lines, collapse = "\n")

  expect_true(grepl("\\*\\*\\[diagnosticsCode\\]\\(diagnosticsCode/\\)\\*\\*", readme_text))
  expect_true(grepl("\\*\\*\\[diagnosticsShiny\\]\\(diagnosticsShiny/\\)\\*\\*", readme_text))
  expect_true(grepl("\\*\\*\\[studyCode\\]\\(studyCode/\\)\\*\\*", readme_text))
  expect_true(grepl("\\*\\*\\[studyShiny\\]\\(studyShiny/\\)\\*\\*", readme_text))

  expect_false(grepl("\\*\\*\\[(diagnosticsCode|diagnosticsShiny|studyCode|studyShiny)/\\]", readme_text))
})

test_that("reviewStudy summarises code files", {
  temp_dir <- file.path(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir, recursive = TRUE)
  initStudy(directory = temp_dir)
  expect_no_error(
    reviewStudy(file.path(temp_dir, "diagnosticsCode"), code = TRUE, dependencies = FALSE)
  )
})

test_that("reviewStudy summarises dependencies", {
  skip_if_not_installed("renv")

  temp_dir <- file.path(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir, recursive = TRUE)
  initStudy(directory = temp_dir)

  # no renv yet — should warn that renv.lock is missing
  expect_warning(
    reviewStudy(file.path(temp_dir, "diagnosticsCode"), code = FALSE, dependencies = TRUE)
  )

  renv::init(file.path(temp_dir, "diagnosticsCode"),
    restart = FALSE,
    load = FALSE
  )
  expect_no_error(
    reviewStudy(file.path(temp_dir, "diagnosticsCode"), code = FALSE, dependencies = TRUE)
  )
})

test_that("reviewStudy works for both code and dependencies", {
  skip_if_not_installed("renv")

  temp_dir <- file.path(tempdir(), omopgenerics::uniqueTableName())
  dir.create(path = temp_dir, recursive = TRUE)
  initStudy(directory = temp_dir)

  renv::init(file.path(temp_dir, "diagnosticsCode"),
    restart = FALSE,
    load = FALSE
  )
  expect_no_error(
    reviewStudy(file.path(temp_dir, "diagnosticsCode"), code = TRUE, dependencies = TRUE)
  )
})

# Docker Wrapper Functions for OMOP Studies

#' Execute a Docker command
#' @param args Character vector of Docker command arguments
#' @param error_message Custom error message if command fails
#' @return Command output as character vector (returned invisibly to avoid console clutter)
#' @keywords internal
dockerExec <- function(args, error_message = "Docker command failed") {
  result <- suppressWarnings(system2("docker", args, stdout = TRUE, stderr = TRUE))
  if (!is.null(attr(result, "status")) && attr(result, "status") != 0) {
    stop(error_message, "\n", paste(result, collapse = "\n"), call. = FALSE)
  }
  return(invisible(result))
}

#' Run a Docker query command and return cleaned output
#' @param args Character vector of Docker command arguments
#' @return Output lines, or empty if the command fails
#' @keywords internal
dockerQuery <- function(args) {
  out <- suppressWarnings(system2("docker", args, stdout = TRUE, stderr = TRUE))
  status <- attr(out, "status")
  if (!is.null(status) && !identical(as.integer(status), 0L)) return(character(0))
  out[nzchar(out)]
}

#' Stop a running OmopStudyBuilder container (automated or RStudio)
#'
#' Stops containers started by \code{runStudy()} and/or \code{runRStudio()}.
#' By default it stops containers for the image inferred from the current
#' directory name.
#'
#' @param container Optional container name or ID to stop directly.
#' @param image_name Optional Docker image name. Defaults to current directory name.
#' @param mode Which container(s) to stop: "any" (default), "rstudio", or "run".
#' @param all If TRUE, stops all running containers started by OmopStudyBuilder.
#' @return TRUE if at least one container was stopped (invisibly)
#' @export
stopStudy <- function(container = NULL, image_name = NULL, mode = c("any", "rstudio", "run"), all = FALSE) {
  ensureDocker()
  mode <- match.arg(mode)
  
  if (!is.null(container) && nzchar(container)) {
    message("Stopping container...")
    dockerExec(c("stop", container), paste0("Failed to stop container: ", container))
    message("Container stopped: ", substr(container, 1, 12))
    return(invisible(TRUE))
  }
  
  if (!isTRUE(all)) {
    image_name <- autoDetectImageName(image_name)
  }
  
  filters <- c("-f", "label=omopstudybuilder=true")
  if (!isTRUE(all)) filters <- c(filters, "-f", paste0("label=omopstudybuilder.image=", image_name))
  if (mode != "any") filters <- c(filters, "-f", paste0("label=omopstudybuilder.mode=", mode))

  ids <- dockerQuery(c("ps", "-q", filters))
  
  if (length(ids) == 0 && !isTRUE(all) && mode %in% c("any", "rstudio")) {
    ids <- dockerQuery(c("ps", "-q", "-f", paste0("ancestor=", image_name)))
  }
  
  if (length(ids) == 0) {
    stop("No running OmopStudyBuilder containers found",
         if (!isTRUE(all)) paste0(" for image: ", image_name) else "", call. = FALSE)
  }
  
  message("Stopping ", length(ids), " container(s)...")
  for (id in ids) dockerExec(c("stop", id), paste0("Failed to stop: ", id))
  message("Stopped.")
  return(invisible(TRUE))
}

#' Check if Docker daemon is running
#' @return TRUE if Docker is available, throws error otherwise
#' @keywords internal
ensureDocker <- function() {
  docker_bin <- Sys.which("docker")
  if (!nzchar(docker_bin)) {
    stop(
      "Docker CLI not found on PATH.\n",
      "Install Docker Desktop (or ensure 'docker' is on PATH) and try again.",
      call. = FALSE
    )
  }

  result <- suppressWarnings(system2(docker_bin, "info", stdout = TRUE, stderr = TRUE))
  status <- attr(result, "status")
  if (is.null(status)) status <- 0L

  if (!identical(as.integer(status), 0L)) {
    stop(
      "Docker daemon is not running.\n",
      "Please start Docker Desktop and try again.\n\n",
      paste(result, collapse = "\n"),
      call. = FALSE
    )
  }

  return(TRUE)
}

#' Build a Docker image for an OMOP study
#' @param image_name Name for the Docker image (default: auto-detected from directory)
#' @param path Path to study directory (default: current directory)
#' @param useRStudio Use RStudio Server base (TRUE) or r-ver base (FALSE, default)
#' @param r_version R version override (default: auto-detected from renv.lock)
#' @param snapshot Update renv.lock before building (default: TRUE)
#' @param github_token Optional GitHub token for installing GitHub packages during build
#' @return Image name (invisibly - already printed to console)
#' @export
dockeriseStudy <- function(image_name = NULL, 
                          path = ".", 
                          useRStudio = FALSE,
                          r_version = NULL,
                          snapshot = TRUE,
                          github_token = NULL) {
  # Ensure Docker is running
  ensureDocker()

  rlang::check_installed("renv", reason = "for dockeriseStudy()")
  rlang::check_installed("jsonlite", reason = "for dockeriseStudy()")
  
  # Update renv.lock to capture all dependencies
  if (snapshot) {
    message("Updating renv.lock (project dependencies only)...")
    # Check for and repair broken symlinks in renv cache
    message("Checking renv integrity...")
    suppressWarnings(suppressMessages(try(renv::repair(project = path), silent = TRUE)))
    
    # Check for missing packages and auto-install them
    deps <- try(renv::dependencies(root = path), silent = TRUE)
    if (inherits(deps, "try-error")) deps <- NULL
    if (!is.null(deps) && nrow(deps) > 0) {
      required_pkgs <- unique(deps$Package)
      installed_pkgs <- rownames(utils::installed.packages())
      missing_pkgs <- setdiff(required_pkgs, installed_pkgs)
      
      if (length(missing_pkgs) > 0) {
        message("Found ", length(missing_pkgs), " missing package(s): ", paste(missing_pkgs, collapse = ", "))
        
        # Try to install each package, but don't fail if some can't install on host
        # (database drivers like odbc/RPostgres often fail on Mac but work fine in Docker)
        for (pkg in missing_pkgs) {
          message("Attempting to install ", pkg, "...")
          install_res <- try(renv::install(pkg, prompt = FALSE), silent = TRUE)
          if (inherits(install_res, "try-error")) {
            cond <- attr(install_res, "condition")
            msg <- if (!is.null(cond) && nzchar(cond$message)) cond$message else as.character(install_res)
            message("  Warning: Could not install ", pkg, " on host machine (", msg, ")")
            message("  This is OK if ", pkg, " is only needed inside Docker (e.g., database drivers)")
          }
        }
      }
    }
    
    # Snapshot all dependencies (non-interactive, force include all dependencies)
    old_interactive <- getOption("renv.config.auto.snapshot")
    options(renv.config.auto.snapshot = FALSE)
    on.exit(options(renv.config.auto.snapshot = old_interactive), add = TRUE)

    snapshot_res <- try({
      # Force snapshot even if some packages couldn't install on host
      # Use type = "all" to include all dependencies found in code, regardless of installation status
      renv::snapshot(project = path, type = "all", prompt = FALSE, force = TRUE)
      message("renv.lock updated successfully")
      
      # Quick status check (don't halt on inconsistencies)
      status_msg <- try(utils::capture.output(renv::status(project = path)), silent = TRUE)
      if (inherits(status_msg, "try-error")) status_msg <- character(0)
      if (length(status_msg) > 0 && any(grepl("inconsistent", status_msg, ignore.case = TRUE))) {
        message("Note: Some packages couldn't install on your host machine but are in renv.lock for Docker to install.")
      }
      TRUE
    }, silent = TRUE)

    if (inherits(snapshot_res, "try-error")) {
      cond <- attr(snapshot_res, "condition")
      msg <- if (!is.null(cond) && nzchar(cond$message)) cond$message else as.character(snapshot_res)
      stop("Failed to snapshot dependencies: ", msg, call. = FALSE)
    }
  }

  # Verify required renv files exist
  required_files <- c("renv.lock", ".Rprofile", file.path("renv", "activate.R"))
  missing <- required_files[!file.exists(file.path(path, required_files))]
  if (length(missing) > 0) {
    stop(
      "Missing required renv files: ", paste(missing, collapse = ", "), "\n",
      "Initialize renv with: renv::init()",
      call. = FALSE
    )
  }

  # If the lockfile includes GitHub packages, Docker builds may require a token
  # (rate limits / restricted network environments).
  lock_file <- file.path(path, "renv.lock")
  lock_data <- try(jsonlite::read_json(lock_file), silent = TRUE)
  if (inherits(lock_data, "try-error")) {
    stop("Could not read renv.lock", call. = FALSE)
  }

  if (is.null(github_token) || !nzchar(github_token)) {
    pkgs <- NULL
    if (is.list(lock_data)) pkgs <- lock_data$Packages
    if (is.list(pkgs) && length(pkgs) > 0) {
      has_github <- any(vapply(pkgs, function(rec) {
        src <- rec[["Source"]]
        isTRUE(identical(src, "GitHub"))
      }, logical(1)))
      if (isTRUE(has_github) && !nzchar(Sys.getenv("GITHUB_PAT"))) {
        warning(
          "renv.lock contains GitHub packages. If Docker build fails downloading from GitHub, ",
          "provide github_token=... (or set GITHUB_PAT) when calling dockeriseStudy().",
          call. = FALSE
        )
      }
    }
  }
  
  # Auto-detect R version from renv.lock if not provided
  if (is.null(r_version)) {
    r_version <- lock_data$R$Version
    if (is.null(r_version)) {
      stop("Could not find R version in renv.lock", call. = FALSE)
    }
    message("Detected R version from renv.lock: ", r_version)
  }
  
  if (!is.null(image_name) && nzchar(image_name)) {
    if (grepl("[A-Z\\s]", image_name)) {
      stop("Invalid image_name: must be lowercase and contain no spaces.", call. = FALSE)
    }
  }
  
  # Ensure .dockerignore exists (prevents copying large/unnecessary files)
  dockerignore_path <- file.path(path, ".dockerignore")
  if (!file.exists(dockerignore_path)) {
    writeLines(c(
      "# RStudio / R artifacts",
      ".Rproj.user",
      ".Rhistory",
      ".RData",
      ".Ruserdata",
      "*.Rproj",
      "",
      "# OS files",
      "Thumbs.db",
      ".DS_Store",
      "",
      "# Build artifacts",
      "Dockerfile",
      "",
      "# Output directories",
      "results/",
      "",
      "# renv library (rebuilt inside container)",
      "renv/library/",
      "renv/staging/",
      "renv/local/",
      "",
      "# Git",
      ".git",
      ".gitignore"
    ), dockerignore_path)
    message("Generated .dockerignore")
  }
  
  # Auto-detect image name from directory if not provided
  if (is.null(image_name)) {
    image_name <- tolower(basename(normalizePath(path)))
    message("Auto-detected image name: ", image_name)
  }
  
  # Determine base image
  base_type <- if (useRStudio) "rstudio" else "r-ver"
  base_image <- paste0("rocker/", base_type, ":", r_version)
  
  # Generate Dockerfile from template
  template_path <- system.file("docker", "Dockerfile.template", package = "OmopStudyBuilder")
  if (!file.exists(template_path)) {
    stop("Dockerfile.template not found in package", call. = FALSE)
  }
  dockerfile_content <- paste(readLines(template_path), collapse = "\n")
  substitutions <- list(
    BASE_IMAGE = base_image,
    EXPOSE_PORT = if (useRStudio) "\n\n# Expose RStudio Server port\nEXPOSE 8787" else ""
  )
  for (name in names(substitutions)) {
    placeholder <- paste0("{{", name, "}}")
    dockerfile_content <- gsub(placeholder, substitutions[[name]], dockerfile_content, fixed = TRUE)
  }
  
  dockerfile_path <- file.path(path, "Dockerfile")
  
  # Write Dockerfile
  writeLines(dockerfile_content, dockerfile_path)
  message("Generated Dockerfile with base: ", base_image)
  
  # Build the image with streaming output
  message("Building Docker image: ", image_name, ":latest")
  message("This may take some minutes on first build...\n")
  
  # Use system2() for safer argument handling
  build_args_vec <- c("build", "--progress=plain", "-t", paste0(image_name, ":latest"))
  if (!is.null(github_token) && nzchar(github_token)) {
    build_args_vec <- c(build_args_vec, "--build-arg", paste0("GITHUB_PAT=", github_token))
  }
  build_args_vec <- c(build_args_vec, normalizePath(path, winslash = "/", mustWork = TRUE))
  exit_code <- system2("docker", build_args_vec)
  
  if (exit_code != 0) {
    stop("Failed to build image: ", image_name, call. = FALSE)
  }
  
  message("\nSuccessfully built: ", image_name, ":latest")
  
  # Return image name invisibly (already shown in messages)
  return(invisible(image_name))
}

#' Push a Docker image to Docker Hub
#' @param image_name Name of Docker image to push (default: auto-detected from directory)
#' @param repo Docker Hub repository (e.g., "username/repo" or "repo")
#' @param tag Tag to push (default: "latest")
#' @param username Docker Hub username (prompted if NULL)
#' @param password Docker Hub password or token (prompted if NULL, hidden input)
#' @param logout If TRUE, logs out after pushing
#' @return Pushed image reference (invisibly)
#' @export
pushDockerImage <- function(image_name = NULL,
                           repo,
                           tag = "latest",
                           username = NULL,
                           password = NULL,
                           logout = TRUE) {
  if (missing(repo) || !nzchar(repo)) {
    stop("repo is required (e.g., 'username/repo' or 'repo').", call. = FALSE)
  }

  ensureDocker()
  rlang::check_installed("getPass", reason = "for pushDockerImage()")
  rlang::check_installed("processx", reason = "for pushDockerImage()")
  
  image_name <- autoDetectImageName(image_name)
  verifyImageExists(image_name)

  if (is.null(username) || !nzchar(username)) {
    username <- readline("Docker Hub username: ")
  }
  if (!nzchar(username)) stop("Username is required.", call. = FALSE)

  if (is.null(password) || !nzchar(password)) {
    password <- getPass::getPass("Docker Hub password or token: ")
  }
  if (!nzchar(password)) stop("Password or token is required.", call. = FALSE)

  repo_ref <- if (grepl("/", repo, fixed = TRUE)) repo else paste0(username, "/", repo)
  image_ref <- paste0(repo_ref, ":", tag)

  stdin_file <- tempfile("dockerhub-password-")
  on.exit(unlink(stdin_file), add = TRUE)
  writeLines(password, stdin_file, useBytes = TRUE)

  login_res <- suppressWarnings(system2(
    "docker",
    c("login", "--username", username, "--password-stdin"),
    stdin = stdin_file,
    stdout = TRUE,
    stderr = TRUE
  ))
  login_status <- attr(login_res, "status")
  if (!is.null(login_status) && !identical(as.integer(login_status), 0L)) {
    stop("Docker login failed.\n", paste(login_res, collapse = "\n"), call. = FALSE)
  }

  dockerExec(c("tag", paste0(image_name, ":latest"), image_ref), "Failed to tag image")
  message("Pushing image (this may take a while): ", image_ref)
  push_res <- processx::run(
    "docker",
    c("push", image_ref),
    echo = TRUE,
    error_on_status = FALSE
  )
  if (!identical(as.integer(push_res$status), 0L)) {
    stop(
      "Failed to push image.\n",
      paste(c(push_res$stdout, push_res$stderr), collapse = "\n"),
      call. = FALSE
    )
  }

  if (isTRUE(logout)) {
    suppressWarnings(system2("docker", "logout", stdout = TRUE, stderr = TRUE))
  }

  message("Pushed image: ", image_ref)
  return(invisible(image_ref))
}

#' Find next available port starting from a given port
#' @param start_port Starting port number to check
#' @param max_tries Maximum number of ports to try
#' @return Available port number
#' @keywords internal
findAvailablePort <- function(start_port = 8787, max_tries = 10) {
  hosts_to_try <- c("127.0.0.1", "localhost", "::1")
  for (i in 0:(max_tries - 1)) {
    port <- start_port + i
    result <- dockerQuery(c("ps", "-q", "--filter", paste0("publish=", port)))
    
    if (length(result) == 0) {
      in_use_local <- FALSE
      for (host in hosts_to_try) {
        con <- suppressWarnings(try(
          socketConnection(host = host, port = port, open = "r+", timeout = 0.25),
          silent = TRUE
        ))
        if (!inherits(con, "try-error")) {
          close(con)
          in_use_local <- TRUE
          break
        }
      }
      if (!isTRUE(in_use_local)) return(port)
    }
  }
  stop("No available ports found between ", start_port, " and ", start_port + max_tries - 1, call. = FALSE)
}

#' Auto-detect image name from current directory
#' @param image_name Existing image name (if provided)
#' @return Image name (lowercased directory basename)
#' @keywords internal
autoDetectImageName <- function(image_name = NULL) {
  if (!is.null(image_name)) {
    return(image_name)
  }
  detected <- tolower(basename(normalizePath(".")))
  message("Auto-detected image name: ", detected)
  return(detected)
}

#' Generate unique Docker container name
#' @param base_name Base name for container
#' @param suffix Optional suffix (e.g., "-run")
#' @param max_tries Maximum attempts to find unique name
#' @return Unique container name
#' @keywords internal
generateContainerName <- function(base_name, suffix = NULL, max_tries = 100) {
  container_base <- if (is.null(suffix)) base_name else paste0(base_name, suffix)
  container_name <- container_base
  for (i in 0:(max_tries - 1)) {
    check <- dockerQuery(c("ps", "-aq", "-f", paste0("name=^", container_name, "$")))
    if (length(check) == 0) return(container_name)
    if (i < max_tries - 1) {
      container_name <- paste0(container_base, "-", sprintf("%02d", i + 1))
    }
  }
  stop("Could not find available container name after ", max_tries, " attempts", call. = FALSE)
}

#' Process environment file argument
#' @param env_file Path to .env file or NULL
#' @param default_env_file Default env file path to use when env_file is NULL
#' @return Character vector of Docker env-file arguments (empty if no file)
#' @keywords internal
processEnvFile <- function(env_file, default_env_file = ".env") {
  if (is.null(env_file)) {
    if (is.null(default_env_file) || !nzchar(default_env_file) || !file.exists(default_env_file)) {
      return(character(0))
    }
    env_file <- default_env_file
  }
  if (!file.exists(env_file)) stop(".env file not found: ", env_file, call. = FALSE)
  message("Using env file: ", env_file)
  return(c("--env-file", normalizePath(env_file, winslash = "/", mustWork = TRUE)))
}

#' Build Docker labels for OmopStudyBuilder containers
#' @param image_name Docker image name
#' @param mode Container mode ("rstudio" or "run")
#' @return Character vector of Docker label arguments
#' @keywords internal
buildDockerLabels <- function(image_name, mode) {
  return(c(
    "--label", "omopstudybuilder=true",
    "--label", paste0("omopstudybuilder.image=", image_name),
    "--label", paste0("omopstudybuilder.mode=", mode)
  ))
}

#' Verify Docker image exists and provide helpful error
#' @param image_name Name of Docker image
#' @param check_rstudio If TRUE, also verify RStudio Server is present
#' @return TRUE (invisibly) if checks pass, otherwise throws error
#' @keywords internal
verifyImageExists <- function(image_name, check_rstudio = FALSE) {
  inspect_status <- suppressWarnings(try(system2(
    "docker",
    c("image", "inspect", image_name),
    stdout = FALSE,
    stderr = FALSE
  ), silent = TRUE))
  if (inherits(inspect_status, "try-error")) inspect_status <- 1L
  if (!isTRUE(inspect_status == 0)) {
    stop("Image '", image_name, "' not found.\nBuild it first with: dockeriseStudy()",
         if (check_rstudio) "\nFor RStudio mode, build with: dockeriseStudy(useRStudio = TRUE)" else "",
         call. = FALSE)
  }
  if (check_rstudio) {
    rserver_status <- suppressWarnings(try(system2(
      "docker",
      c("run", "--rm", "--entrypoint", "/bin/sh", image_name,
        "-c", "command -v rserver >/dev/null 2>&1"),
      stdout = FALSE,
      stderr = FALSE
    ), silent = TRUE))
    if (inherits(rserver_status, "try-error")) rserver_status <- 1L

    if (!isTRUE(rserver_status == 0)) {
      stop(
        "RStudio Server not available in image '", image_name, "'.\n",
        "Rebuild with: dockeriseStudy(useRStudio = TRUE)",
        call. = FALSE
      )
    }
  }
  return(invisible(TRUE))
}

#' Run RStudio Server for interactive study execution
#'
#' Note: The Docker image must include RStudio Server (i.e., be built with
#' `dockeriseStudy(useRStudio = TRUE)` which uses a `rocker/rstudio` base image).
#' @param image_name Name of Docker image to run (default: auto-detected from directory)
#' @param results_path Path to save results (default: "./results")
#' @param env_file Optional path to a .env file (passed to Docker via --env-file).
#'   If NULL and a `.env` file exists in the current working directory, it will be
#'   used automatically.
#' @param port Port for RStudio Server (default: 8787, auto-finds next available if busy)
#' @param password RStudio password (default: auto-generated and displayed)
#' @return Container ID (invisibly)
#' @importFrom utils browseURL
#' @export
runRStudio <- function(image_name = NULL, results_path = "./results", env_file = NULL,
                       port = 8787, password = NULL) {
  ensureDocker()
  image_name <- autoDetectImageName(image_name)
  verifyImageExists(image_name, check_rstudio = TRUE)
  
  original_port <- port
  port <- findAvailablePort(start_port = port)
  if (port != original_port) message("Port ", original_port, " in use, using port ", port, " instead")
  
  if (is.null(password)) password <- paste0("study", sample(1000:9999, 1))
  if (!dir.exists(results_path)) dir.create(results_path, recursive = TRUE)
  
  env_args <- processEnvFile(env_file)
  container_name <- generateContainerName(image_name)
  labels <- buildDockerLabels(image_name, "rstudio")
  
  args <- c("run", "-d", "--rm", "--name", container_name, "-p", paste0(port, ":8787"),
            labels, env_args, "-e", paste0("PASSWORD=", password),
            "-v", paste0(normalizePath(results_path), ":/home/rstudio/study/results"), image_name)
  
  message("Starting RStudio Server...")
  container_id <- dockerExec(args, "Failed to start RStudio Server")[1]
  check <- dockerQuery(c("ps", "-q", "-f", paste0("id=", container_id)))
  if (length(check) == 0) {
    stop("Container exited unexpectedly.\nCheck logs with: docker logs ", container_id, call. = FALSE)
  }
  
  hosts_to_try <- c("127.0.0.1", "localhost", "::1")
  reachable_host <- NULL
  start <- Sys.time()
  while (is.null(reachable_host) && as.numeric(difftime(Sys.time(), start, units = "secs")) < 30) {
    for (host in hosts_to_try) {
      con <- suppressWarnings(try(socketConnection(host = host, port = port, open = "r+", timeout = 0.5), silent = TRUE))
      if (!inherits(con, "try-error")) {
        close(con)
        reachable_host <- host
        break
      }
    }
    if (is.null(reachable_host)) Sys.sleep(0.5)
  }
  if (is.null(reachable_host)) {
    logs <- try(system2("docker", c("logs", "--tail", "200", container_id), stdout = TRUE, stderr = TRUE), silent = TRUE)
    if (inherits(logs, "try-error")) {
      cond <- attr(logs, "condition")
      msg <- if (!is.null(cond) && nzchar(cond$message)) cond$message else as.character(logs)
      logs <- paste0("(Failed to read logs: ", msg, ")")
    }
    stop("RStudio container started but not listening on port ", port, " after 30s.\n\n",
         "Last 200 log lines:\n", paste(logs, collapse = "\n"), call. = FALSE)
  }
  
  url_host <- if (grepl(":", reachable_host, fixed = TRUE)) paste0("[", reachable_host, "]") else reachable_host
  url <- paste0("http://", url_host, ":", port)
  message("\nRStudio Server started successfully!\n\n  URL:      ", url, "\n  Username: rstudio",
          "\n  Password: ", password, "\n\nResults will be saved to: ", results_path,
          "\nContainer will auto-remove when stopped.\n\nTo stop: docker stop ", substr(container_id, 1, 12))

  browse_res <- try(browseURL(url), silent = TRUE)
  if (inherits(browse_res, "try-error")) {
    message("\nCould not open browser automatically.\nPlease open manually: ", url)
  } else {
    message("\nOpening browser...")
  }
  
  return(invisible(container_id))
}

#' Run study in automated mode with real-time log streaming
#' @param image_name Name of Docker image to run (default: auto-detected from directory)
#' @param results_path Path to save results (default: "./results")
#' @param env_file Optional path to a .env file (passed to Docker via --env-file).
#'   If NULL and a `.env` file exists in the current working directory, it will be
#'   used automatically.
#' @param data_path Optional path to data directory (mounted at /data)
#' @param script_path Path to R script to execute (default: "codeToRun.R")
#' @return Exit status (0 = success, non-zero = failure)
#' @export
runStudy <- function(image_name = NULL, results_path = "./results", env_file = NULL,
                     data_path = NULL, script_path = "codeToRun.R") {
  ensureDocker()
  rlang::check_installed("processx", reason = "for runStudy()")
  
  image_name <- autoDetectImageName(image_name)
  verifyImageExists(image_name)
  
  if (!dir.exists(results_path)) dir.create(results_path, recursive = TRUE)
  env_args <- processEnvFile(env_file)
  
  mounts <- c("-v", paste0(normalizePath(results_path), ":/workspace/results"))
  if (!is.null(data_path)) {
    if (!dir.exists(data_path)) stop("Data directory not found: ", data_path, call. = FALSE)
    mounts <- c(mounts, "-v", paste0(normalizePath(data_path), ":/data"))
    message("Mounting data: ", data_path)
  }
  
  container_name <- generateContainerName(image_name, suffix = "-run")
  labels <- buildDockerLabels(image_name, "run")
  args <- c("run", "--rm", "--name", container_name, labels, env_args, mounts, 
            image_name, "Rscript", script_path)
  
  message("\n========================================\nStarting automated study execution...\nContainer: ", 
          container_name, "\nScript: ", script_path, "\nResults: ", results_path, 
          "\n========================================\n")
  
  proc <- processx::process$new("docker", args, stdout = "|", stderr = "|", 
                                echo_cmd = FALSE, cleanup_tree = TRUE)
  
  while (proc$is_alive()) {
    if (proc$is_incomplete_output()) {
      output <- proc$read_output(n = 1000)
      if (length(output) > 0 && output != "") cat(output)
    }
    if (proc$is_incomplete_error()) {
      errors <- proc$read_error(n = 1000)
      if (length(errors) > 0 && errors != "") cat(errors, file = stderr())
    }
    proc$poll_io(100)
    Sys.sleep(0.05)
  }
  
  remaining_out <- proc$read_all_output()
  if (length(remaining_out) > 0 && remaining_out != "") cat(remaining_out)
  remaining_err <- proc$read_all_error()
  if (length(remaining_err) > 0 && remaining_err != "") cat(remaining_err, file = stderr())
  
  exit_status <- proc$get_exit_status()
  message("\n========================================")
  if (exit_status == 0) {
    message("Study completed successfully!\nResults saved to: ", results_path)
  } else {
    message("Study failed with exit code: ", exit_status, "\nCheck logs above for errors")
  }
  message("========================================\n")
  return(invisible(exit_status))
}

library(testthat)

# -------------------------------------------------------------------------
# Docker Detection
# -------------------------------------------------------------------------

test_that("ensureDocker detects Docker daemon", {
  skip_if_not(system2("docker", "info", stdout = FALSE, stderr = FALSE) == 0,
              "Docker not available")
  
  expect_true(ensureDocker())
})



# -------------------------------------------------------------------------
# pushDockerImage Tests
# -------------------------------------------------------------------------

test_that("pushDockerImage requires repo", {
  skip_if_not(exists("pushDockerImage", mode = "function"), "Package not loaded")
  expect_error(
    pushDockerImage(),
    "repo is required",
    fixed = TRUE
  )
})

test_that("pushDockerImage errors when image is missing", {
  skip_if_not(system2("docker", "info", stdout = FALSE, stderr = FALSE) == 0,
              "Docker not available")

  expect_error(
    pushDockerImage(
      image_name = "__definitely_not_an_image__",
      repo = "someone/somewhere",
      username = "x",
      password = "y",
      logout = FALSE
    ),
    "not found",
    ignore.case = TRUE
  )
})



# -------------------------------------------------------------------------
# dockeriseStudy Tests
# -------------------------------------------------------------------------

test_that("dockeriseStudy uses r-ver by default", {
  skip("Integration test - requires valid renv project")
})

test_that("dockeriseStudy uses rstudio when requested", {
  skip("Integration test - requires valid renv project")
})

# -------------------------------------------------------------------------
# runRStudio Tests
# -------------------------------------------------------------------------

test_that("runRStudio auto-detects image name", {
  skip("Integration test - requires built rstudio image")
})

test_that("runRStudio generates password if not provided", {
  skip("Integration test - requires built rstudio image")
})

test_that("runRStudio creates results directory", {
  skip("Integration test - requires built rstudio image")
})

test_that("runRStudio returns container ID invisibly", {
  skip("Integration test - requires built rstudio image")
})

# -------------------------------------------------------------------------
# Container Lifecycle Tests
# -------------------------------------------------------------------------

test_that("stopStudy stops rstudio containers", {
  skip("Integration test - requires running container")
})

test_that("stopStudy accepts explicit container name", {
  skip("Integration test - requires running container")
})



# -------------------------------------------------------------------------
# Automated Execution Tests
# -------------------------------------------------------------------------

test_that("runStudy requires processx package", {
  skip("Integration test - requires built image")
})

test_that("runStudy auto-detects image name", {
  skip("Integration test - requires built image and test script")
})

test_that("runStudy mounts config file when provided", {
  skip("Integration test - requires built image and config file")
})

test_that("runStudy streams logs in real-time", {
  skip("Integration test - requires built image and test script")
})

test_that("runStudy returns exit status", {
  skip("Integration test - requires built image and test script")
})

#' Review a study directory
#'
#' Summarises the code and/or renv dependencies in a study directory.
#'
#' @param dir Path to the study directory.
#' @param code If `TRUE`, summarises R, JSON, CSV, and Excel files found in
#'   the directory.
#' @param dependencies If `TRUE`, summarises the renv.lock dependencies.
#' @param type Whether the R project is for analysis code or study reporting.
#'   Only used when `dependencies = TRUE`.
#'
#' @returns Invisibly returns `NULL`. Called for its side effects of printing
#'   summaries to the console.
#' @export
#'
reviewStudy <- function(dir, code = TRUE, dependencies = TRUE, type = "analysis") {

  dir <- normalizePath(dir, mustWork = TRUE)

  if (isTRUE(dependencies)) {
    omopgenerics::assertChoice(type, choices = c("analysis", "reporting"))
    rlang::check_installed("renv", reason = "for reviewing dependencies")
    rlang::check_installed("dplyr", reason = "for reviewing dependencies")
    rlang::check_installed("purrr", reason = "for reviewing dependencies")
  }
  
  if (isTRUE(code)) {
    rlang::check_installed("stringr", reason = "for reviewing code")
    rlang::check_installed("scales", reason = "for reviewing code")
    rlang::check_installed("purrr", reason = "for reviewing code")
    
    cli::cli_h1("Code Summary")
    summariseRFiles(dir)
    summariseJsonFiles(dir)
    summariseCsvFiles(dir)
    summariseExcelFiles(dir)
  }

  if (isTRUE(dependencies)) {
    cli::cli_h1("Dependencies")

    if (!file.exists(file.path(dir, "renv.lock"))) {
      cli::cli_warn(c("renv.lock file not found in {dir}",
                      i = "A renv file should be added to ensure reproducibility"))
      return(invisible(NULL))
    }

    reportDependencies(dir)
    lock <- renv::lockfile_read(file.path(dir, "renv.lock"))
    reportLockPkgs(lock, type)
    checkPkgSource(lock)
    checkLatestVersion(lock)
  }

  return(invisible(NULL))
}

# =============================================================================
# Code file helpers
# =============================================================================

summariseRFiles <- function(dir) {
  cli::cat_line()
  cli::cli_h2("R scripts")

  rFiles <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.R$",
    ignore.case = TRUE,
    full.names = TRUE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  rFilesShort <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.R$",
    ignore.case = TRUE,
    full.names = FALSE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  rFilesShortLoc <- paste0(
    rFilesShort,
    " (loc: ",
    purrr::map_int(rFiles, ~ get_loc(.x)) |> scales::comma(),
    ")"
  )
  totalLoc <- sum(purrr::map_int(rFiles, ~ get_loc(.x))) |> scales::comma()

  cli::cli_inform(c(
    "{dir} contains {length(rFiles)} R scripts (total lines of code {totalLoc})",
    i = "{rFilesShortLoc}"
  ))
}

summariseJsonFiles <- function(dir) {
  cli::cat_line()
  cli::cli_h2("JSON files")

  jsonFiles <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.json$",
    ignore.case = TRUE,
    full.names = TRUE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  jsonFilesShort <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.json$",
    ignore.case = TRUE,
    full.names = FALSE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  cli::cli_inform(c(
    "{dir} contains {length(jsonFiles)} JSON files",
    i = "{jsonFilesShort}"
  ))
}

summariseCsvFiles <- function(dir) {
  cli::cat_line()
  cli::cli_h2("CSV files")

  csvFiles <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.csv$",
    ignore.case = TRUE,
    full.names = TRUE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  csvFilesShort <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.csv$",
    ignore.case = TRUE,
    full.names = FALSE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  cli::cli_inform(c(
    "{dir} contains {length(csvFiles)} CSV files",
    i = "{csvFilesShort}"
  ))
}

summariseExcelFiles <- function(dir) {
  cli::cat_line()
  cli::cli_h2("Excel files")

  excelFiles <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.xlsx$|\\.xls$",
    ignore.case = TRUE,
    full.names = TRUE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  excelFilesShort <- list.files(dir,
    recursive = TRUE,
    pattern = "\\.xlsx$|\\.xls$",
    ignore.case = TRUE,
    full.names = FALSE
  ) |>
    stringr::str_subset(pattern = "renv/", negate = TRUE)

  cli::cli_inform(c(
    "{dir} contains {length(excelFiles)} Excel files",
    i = "{excelFilesShort}"
  ))
}

get_loc <- function(file_path) {
  length(readLines(file_path, warn = FALSE))
}

# =============================================================================
# Dependency helpers
# =============================================================================

reportDependencies <- function(dir) {
  cli::cat_line()
  cli::cli_h2("Dependencies used in code")

  directDependencies <- renv::dependencies(dir, quiet = TRUE) |>
    dplyr::pull("Package") |>
    unique() |>
    sort()
  directDependenciesChr <- paste0(directDependencies, collapse = "; ")

  cli::cli_inform(c(
    "In total, {length(directDependencies)} packages are used in the code.",
    i = "{directDependenciesChr}"
  ))

  return(invisible())
}

reportLockPkgs <- function(lock, type) {
  cli::cat_line()
  cli::cli_h2("Packages in renv lock file")

  lockPkgs <- names(lock$Packages) |> sort()
  lockPkgsVer <- lockPkgs |>
    purrr::map_chr(~ {
      workingPkg <- .x
      paste0(workingPkg, " (", lock$Packages[[workingPkg]]$Version, ")")
    })
  lockPkgsChr <- paste0(lockPkgsVer, collapse = "; ")
  cli::cli_inform(c(
    "The renv.lock file contains {length(lockPkgs)} packages",
    i = "{lockPkgsChr}"
  ))

  if (type == "analysis") {
    warnJavaPkgs <- c("SqlRender", "CirceR")
    if (any(warnJavaPkgs %in% lockPkgs)) {
      cli::cat_line()
      cli::cli_inform(c(
        "Can {.code {intersect(warnJavaPkgs, lockPkgs)}} be removed?",
        i = "Use requires JAVA installation."
      ))
    }

    warnRVerPkgs <- c("MASS", "Matrix")
    if (any(warnRVerPkgs %in% lockPkgs)) {
      cli::cat_line()
      cli::cli_inform(c(
        "Can {.code {intersect(warnRVerPkgs, lockPkgs)}} be removed?",
        i = "Packages tied to a specific version of R and can be challenging for data partners to restore."
      ))
    }

    warnVisPkgs <- c("ggplot2", "gt", "flextable", "datatable", "reactable", "tinytable", "plotly")
    if (any(warnVisPkgs %in% lockPkgs)) {
      cli::cat_line()
      cli::cli_inform(c(
        "Can {.code {intersect(warnVisPkgs, lockPkgs)}} be removed?",
        i = "Reporting packages should not need to be installed by a data partner to run analysis code."
      ))
    }
  }

  return(invisible())
}

checkPkgSource <- function(lock) {
  cli::cat_line()
  cli::cli_h2("Checking package sources")

  pkgs <- names(lock$Packages) |> sort()
  sources <- purrr::map_dfr(pkgs, ~ dplyr::tibble(
    pkg    = .x,
    source = lock$Packages[[.x]]$Source
  ))

  unknownPkgs <- sources |>
    dplyr::filter(.data$source == "unknown") |>
    dplyr::pull("pkg")

  if (length(unknownPkgs) > 0) {
    cli::cli_inform(c(
      "{length(unknownPkgs)} packages come from an unknown source",
      i = "{unknownPkgs}"
    ))
  } else {
    cli::cli_inform("All packages are from a known source")
  }

  return(invisible())
}

checkLatestVersion <- function(lock) {
  cli::cat_line()
  cli::cli_h2("Checking package versions")

  packages <- names(lock$Packages) |>
    sort() |>
    purrr::map_dfr(~ dplyr::tibble(
      pkg          = .x,
      renv_version = lock$Packages[[.x]]$Version
    ))

  packages <- packages |>
    dplyr::left_join(
      utils::available.packages(repos = "https://cran.rstudio.com/") |>
        dplyr::as_tibble() |>
        dplyr::select("pkg" = "Package", "cran_version" = "Version"),
      by = "pkg"
    )

  discordantPackages <- packages |>
    dplyr::filter(.data$renv_version != .data$cran_version) |>
    dplyr::mutate(compare = paste0(
      .data$pkg, " ", .data$renv_version,
      " (renv) vs ", .data$cran_version, " (cran)"
    ))

  if (nrow(discordantPackages) > 0) {
    cli::cli_inform(c(
      "{length(discordantPackages$pkg)} packages do not correspond to the current CRAN version",
      i = "{discordantPackages$compare}"
    ))
  } else {
    cli::cli_inform("All packages are from latest cran version")
  }

  return(invisible())
}

# GitHub Integration for OMOP Studies

#' Link study directory to GitHub repository
#'
#' Creates a new GitHub repository and connects it to an existing study directory.
#' Initializes git, creates .gitignore, commits all files, and pushes to GitHub.
#' 
#' @section Requirements:
#' \itemize{
#'   \item \strong{GitHub authentication}: Set up via GITHUB_PAT, gh CLI, or git credentials
#'   \item \strong{R package 'gh'}: Install with \code{install.packages("gh")}
#'   \item \strong{R package 'gert'}: Install with \code{install.packages("gert")}
#' }
#' 
#' @section Authentication:
#' This function needs credentials for both:
#' \itemize{
#'   \item \strong{GitHub API access}: used by \code{gh} to check your account and create the repository
#'   \item \strong{Git transport authentication}: used by \code{gert} to push the local repository to GitHub
#' }
#' 
#' Recommended setup:
#' \itemize{
#'   \item \strong{GITHUB_PAT environment variable}: recommended for HTTPS authentication and works for both GitHub API calls and Git pushes to GitHub
#'   \item \strong{Stored Git credentials}: credentials in your OS credential store or Git credential helper can also work for both the API and push steps
#'   \item \strong{gh CLI}: may help set up GitHub authentication, but you may still need Git credentials available for the final push
#'   \item \strong{SSH}: supported when your remote/auth setup is configured accordingly
#' }
#' 
#' To create a Personal Access Token (PAT):
#' \enumerate{
#'   \item Visit \url{https://github.com/settings/tokens}
#'   \item Generate a token with \code{repo} scope
#'   \item Set for current session: \code{Sys.setenv(GITHUB_PAT = "your_token_here")}
#'   \item Or add to .Renviron: \code{GITHUB_PAT='your_token_here'} and restart R
#' }
#'
#' @param directory Path to study directory
#' @param repository GitHub repository name (will be sanitised if needed)
#' @param organisation GitHub organisation name. If NULL (default), creates repo
#'   under your personal account
#' @param private Logical. If TRUE (default), creates a private repository
#' @param description Repository description. If NULL, auto-generated from directory name
#'
#' @return GitHub repository URL (invisibly)
#' @export
#'
#' @examples
#' \dontrun{
#' library(OmopStudyBuilder)
#' library(here)
#' 
#' # Authenticate (choose one method):
#' # 1. Set GITHUB_PAT for current session
#' Sys.setenv(GITHUB_PAT = "your_token_here")
#' # 2. Or use gh CLI: gh auth login
#' 
#' # Create repo under personal account
#' linkGitHub(
#'   directory = here::here(),
#'   repository = "my-omop-study"
#' )
#'
#' # Create repo under organisation
#' linkGitHub(
#'   directory = here::here(),
#'   repository = "diabetes-study",
#'   organisation = "oxford-pharmacoepi",
#'   private = TRUE
#' )
#' }
linkGitHub <- function(directory,
                       repository,
                       organisation = NULL,
                       private = TRUE,
                       description = NULL) {
  # Validate inputs
  omopgenerics::assertCharacter(directory, length = 1)
  omopgenerics::assertCharacter(repository, length = 1, minNumCharacter = 1)
  omopgenerics::assertCharacter(organisation, length = 1, null = TRUE)
  omopgenerics::assertLogical(private, length = 1)
  omopgenerics::assertCharacter(description, length = 1, null = TRUE)
  
  if (!dir.exists(directory)) {
    cli::cli_abort("Directory does not exist: {.path {directory}}")
  }
  directory <- normalizePath(directory, mustWork = TRUE)
  
  # Check required packages are installed
  rlang::check_installed("gh", reason = "for GitHub integration")
  rlang::check_installed("gert", reason = "for local Git operations during GitHub integration")
  
  # Validate repository name
  validateRepoName(repository)
  
  # Check authentication
  cli::cli_alert_info("Checking GitHub authentication...")
  user_info <- checkGitHubAuth()
  owner <- organisation %||% user_info$login
  
  # Check if repo available
  if (!checkRepoAvailable(owner, repository)) {
    cli::cli_abort("Repository {.val {owner}/{repository}} already exists on GitHub")
  }
  
  # Initialize git locally
  cli::cli_alert_info("Initialising git repository...")
  created_repo <- !dir.exists(file.path(directory, ".git"))
  initializeGitRepo(directory)
  
  # Create .gitignore
  createStudyGitIgnore(directory)
  
  # Create GitHub repository
  cli::cli_alert_info("Creating GitHub repository: {.val {owner}/{repository}}")
  if (is.null(description)) {
    description <- paste0("OMOP CDM study: ", basename(directory), " | Created with OmopStudyBuilder")
  }
  repo_info <- createGitHubRepo(repository, organisation, private, description)
  repo_url <- repo_info$html_url
  
  # Setup remote and push
  cli::cli_alert_info("Pushing to GitHub...")
  result <- setupGitRemote(
    directory = directory,
    clone_url = repo_info$clone_url,
    user_info = user_info,
    created_repo = created_repo
  )
  
  # Check if user cancelled
  if (is.null(result)) {
    cli::cli_alert_warning("GitHub repository created but not linked to local directory")
    cli::cli_alert_info("Repository URL: {.url {repo_url}}")
    return(invisible(NULL))
  }
  
  cli::cli_alert_success("Study linked to GitHub: {.url {repo_url}}")
  return(invisible(repo_url))
}


#' Check GitHub authentication
#' @keywords internal
checkGitHubAuth <- function() {
  user <- try(gh::gh_whoami(), silent = TRUE)
  
  if (inherits(user, "try-error")) {
    cli::cli_abort(c(
      "GitHub authentication failed",
      "i" = "See {.code ?linkGitHub} for authentication setup details"
    ))
  }
  
  if (is.null(user) || is.null(user$login)) {
    cli::cli_abort("Failed to fetch GitHub user information")
  }
  
  # gh_whoami() doesn't include id, so fetch complete user info from API
  user_full <- try(gh::gh("GET /user"), silent = TRUE)
  if (!inherits(user_full, "try-error") && !is.null(user_full$id)) {
    user$id <- user_full$id
  }
  
  cli::cli_alert_success("Authenticated as: {.val {user$login}}")
  return(user)
}


#' Check if repository name is available
#' @keywords internal
checkRepoAvailable <- function(owner, repo) {
  # Try to get the repo - if it exists, gh will return the repo info
  # If it doesn't exist, gh will throw a 404 error
  repo_check <- try(gh::gh("GET /repos/{owner}/{repo}", owner = owner, repo = repo), silent = TRUE)
  
  if (inherits(repo_check, "try-error")) {
    error_msg <- attr(repo_check, "condition")$message
    if (grepl("404", error_msg)) {
      return(TRUE)  # Repo doesn't exist (available)
    }
    # If not 404, it's a real error (network issue, auth problem, etc.)
    cli::cli_abort("Failed to check repository availability: {error_msg}")
  }
  
  return(FALSE)  # Repo exists
}


#' Validate repository name for GitHub
#'
#' Checks if a repository name follows GitHub naming rules.
#' Throws an error with helpful suggestions if invalid.
#'
#' @param name Repository name to validate
#' @return TRUE if valid, throws error otherwise
#' @keywords internal
validateRepoName <- function(name) {
  max_length <- 30
  
  # Check length
  if (nchar(name) > max_length) {
    cli::cli_abort(c(
      "Repository name too long: {nchar(name)} characters (max {max_length})",
      "i" = "Shorten to: {.val {substr(name, 1, max_length)}}"
    ))
  }
  
  # Check for spaces
  if (grepl("\\s", name)) {
    cli::cli_abort(c(
      "Repository name cannot contain spaces: {.val {name}}",
      "i" = "Use hyphens instead: {.val {gsub('\\\\s+', '-', name)}}"
    ))
  }
  
  # Check for invalid characters (allow letters, numbers, hyphens, underscores, dots)
  if (grepl("[^a-zA-Z0-9._-]", name)) {
    cli::cli_abort(c(
      "Repository name contains invalid characters: {.val {name}}",
      "i" = "Only letters, numbers, hyphens (-), underscores (_), and periods (.) are allowed"
    ))
  }
  
  # Check leading/trailing hyphens
  if (grepl("^-|-$", name)) {
    cli::cli_abort(c(
      "Repository name cannot start or end with hyphen: {.val {name}}",
      "i" = "Remove leading/trailing hyphens"
    ))
  }
  
  return(invisible(TRUE))
}


#' Create GitHub repository via API
#' @keywords internal
createGitHubRepo <- function(repository, organisation, private, description) {
  if (is.null(organisation)) {
    # Create under user account
    repo <- gh::gh(
      "POST /user/repos",
      name = repository,
      private = private,
      description = description,
      auto_init = FALSE
    )
  } else {
    # Create under organisation
    repo <- gh::gh(
      "POST /orgs/{org}/repos",
      org = organisation,
      name = repository,
      private = private,
      description = description,
      auto_init = FALSE
    )
  }
  
  if (is.null(repo) || is.null(repo$clone_url)) {
    cli::cli_abort(c(
      "Failed to create GitHub repository - incomplete API response",
      "i" = "Expected repository information with clone URL",
      "i" = "This may be a temporary GitHub API issue. Please try again.",
      "i" = "Check GitHub status: {.url https://www.githubstatus.com}"
    ))
  }
  
  return(repo)
}


gertCall <- function(expr, error_message) {
  tryCatch(
    expr,
    error = function(error) {
      cli::cli_abort("{error_message}: {conditionMessage(error)}")
    }
  )
}


getGitConfigValue <- function(directory, key) {
  override <- getOption("OmopStudyBuilder.git_config_getter")
  if (is.function(override)) {
    return(override(directory, key))
  }

  config <- gert::git_config(repo = directory)
  values <- config$value[config$name == tolower(key)]

  if (length(values) == 0 || !nzchar(values[1])) {
    return(NULL)
  }

  values[1]
}


hasStagedChanges <- function(directory) {
  nrow(gert::git_status(staged = TRUE, repo = directory)) > 0
}


#' Initialize local git repository
#' @keywords internal
initializeGitRepo <- function(directory) {
  git_dir <- file.path(directory, ".git")
  
  if (!dir.exists(git_dir)) {
    gertCall(gert::git_init(path = directory), "Failed to initialize git repository")
  }
  
  return(invisible(NULL))
}


#' Create .gitignore for OMOP studies
#' @keywords internal
createStudyGitIgnore <- function(directory) {
  gitignore_path <- file.path(directory, ".gitignore")
  
  gitignore_content <- c(
    "# R artifacts",
    ".Rproj.user",
    ".RData",
    ".Rhistory",
    ".Ruserdata",
    "",
    "# Study outputs (don't commit results data)",
    "results/**/*.csv",
    "",
    "# renv library (only commit lockfile)",
    "renv/library/",
    "renv/staging/",
    "renv/local/",
    "",
    "# OS files",
    "*.DS_Store",
    "Thumbs.db",
    "",
    "# Docker (generated files)",
    "Dockerfile",
    ".dockerignore"
  )
  
  if (file.exists(gitignore_path)) {
    existing <- trimws(readLines(gitignore_path, warn = FALSE))
    # Only add missing lines
    to_add <- setdiff(gitignore_content, existing)
    if (length(to_add) > 0) {
      writeLines(c(existing, "", to_add), gitignore_path)
    }
  } else {
    writeLines(gitignore_content, gitignore_path)
  }
  
  return(invisible(gitignore_path))
}


#' Create git commit
#' @keywords internal
gitCommit <- function(directory, message, allow_nothing_to_commit = FALSE,
                      error_message = "Failed to commit") {
  if (allow_nothing_to_commit && !hasStagedChanges(directory)) {
    return(invisible(FALSE))
  }

  tryCatch(
    {
      gert::git_commit(message = message, repo = directory)
      return(invisible(TRUE))
    },
    error = function(error) {
      if (allow_nothing_to_commit && !hasStagedChanges(directory)) {
        return(invisible(FALSE))
      }

      cli::cli_abort("{error_message}: {conditionMessage(error)}")
    }
  )
}


#' Ensure Git identity is configured
#' 
#' Checks if Git user.name and user.email are configured for commits.
#' If not configured, automatically sets them using GitHub account info
#' provided by the caller. This allows seamless Git operations without
#' requiring manual `git config` setup.
#' 
#' @keywords internal
ensureGitIdentity <- function(directory, user_info = NULL) {
  # Check if already configured
  if (!is.null(getGitConfigValue(directory, "user.name")) &&
      !is.null(getGitConfigValue(directory, "user.email"))) {
    return(invisible(TRUE))
  }
  
  if (is.null(user_info)) {
    cli::cli_abort(c(
      "Git identity not configured and no GitHub user information was provided",
      "i" = "Run: git config --global user.name 'Your Name'",
      "i" = "Run: git config --global user.email 'your@email.com'"
    ))
  }
  
  # Configure user.name (use name or fallback to login)
  if (is.null(getGitConfigValue(directory, "user.name"))) {
    name <- if (!is.null(user_info$name) && nzchar(user_info$name)) user_info$name else user_info$login
    gert::git_config_set("user.name", name, repo = directory)
    cli::cli_alert_info("Configured Git user.name: {.val {name}}")
  }
  
  # Configure user.email (required for commits)
  if (is.null(getGitConfigValue(directory, "user.email"))) {
    if (!is.null(user_info$email) && nzchar(user_info$email)) {
      # Use public email from GitHub
      gert::git_config_set("user.email", user_info$email, repo = directory)
      cli::cli_alert_info("Configured Git user.email: {.val {user_info$email}}")
    } else if (!is.null(user_info$id) && !is.null(user_info$login)) {
      # Construct GitHub noreply email when privacy is enabled
      noreply_email <- paste0(user_info$id, "+", user_info$login, "@users.noreply.github.com")
      gert::git_config_set("user.email", noreply_email, repo = directory)
      cli::cli_alert_info("Configured Git user.email: {.val {noreply_email}}")
    } else {
      # Last resort: fail with helpful message
      cli::cli_abort(c(
        "Git email is required for commits but could not be determined from GitHub",
        "i" = "Configure your Git email globally:",
        "i" = "  git config --global user.email 'your@email.com'",
        "i" = "Or make your GitHub email public in your GitHub settings"
      ))
    }
  }
  
  return(invisible(TRUE))
}


#' Setup git remote and push
#' @keywords internal
setupGitRemote <- function(directory,
                           clone_url,
                           user_info = NULL,
                           created_repo = FALSE) {
  # Ensure local Git identity is configured using caller-provided GitHub info
  ensureGitIdentity(directory, user_info)
  
  # Setup remote (update if exists, add if new)
  remotes <- gert::git_remote_list(repo = directory)
  existing_remote <- remotes$url[remotes$name == "origin"]
  
  if (!is.null(existing_remote) && length(existing_remote) > 0 && nzchar(existing_remote[1])) {
    # Remote exists - ask user before updating
    cli::cli_alert_warning(
      "Directory already linked to: {.url {existing_remote[1]}}"
    )
    cli::cli_alert_warning(
      "This will switch remote to: {.url {clone_url}}"
    )
    
    # Check if running interactively
    if (!interactive()) {
      cli::cli_alert_info("Non-interactive session detected. Remote not changed.")
      cli::cli_alert_info("To update, run interactively or use: git remote set-url origin {clone_url}")
      return(invisible(NULL))
    }
    
    response <- utils::askYesNo("Continue?", default = FALSE)
    if (isFALSE(response)) {
      cli::cli_alert_info("Cancelled. Remote not changed.")
      return(invisible(NULL))
    }
    
    gertCall(
      gert::git_remote_set_url(url = clone_url, remote = "origin", repo = directory),
      "Failed to update git remote"
    )
    cli::cli_alert_success("Remote updated successfully")
  } else {
    # No remote - add it
    gertCall(
      gert::git_remote_add(url = clone_url, name = "origin", repo = directory),
      "Failed to add git remote"
    )
  }
  
  # Stage all files
  gertCall(gert::git_add(files = ".", repo = directory), "Failed to stage files")
  
  # Commit
  commit_msg <- paste0("Initialize study: ", basename(directory))
  committed <- gitCommit(directory, commit_msg, allow_nothing_to_commit = TRUE)
  
  # Check if there are any commits to push
  repo_info <- gert::git_info(repo = directory)
  if (isFALSE(committed) && is.na(repo_info$commit)) {
    # No commits yet - create a README to ensure there's something to commit
    readme_path <- file.path(directory, "README.md")
    if (!file.exists(readme_path)) {
      # Read README template and substitute placeholders
      template_path <- system.file("templates", "README_GITHUB.md", package = "OmopStudyBuilder")
      readme_content <- readLines(template_path, warn = FALSE)
      readme_content <- gsub("{{REPO_NAME}}", basename(directory), readme_content)
      readme_content <- gsub("{{DATE}}", as.character(Sys.Date()), readme_content)
      writeLines(readme_content, readme_path)
      
      # Stage and commit the README
      gertCall(gert::git_add(files = "README.md", repo = directory), "Failed to stage README")
      gitCommit(
        directory,
        "Initialize repository with README",
        error_message = "Failed to create initial commit"
      )
    }
  }
  
  # For new repos, always use "main" branch
  # For existing repos, keep the current branch
  if (isTRUE(created_repo)) {
    current_branch <- gert::git_branch(repo = directory)
    
    if (!identical(current_branch, "main")) {
      gertCall(
        gert::git_branch_move(
          branch = current_branch,
          new_branch = "main",
          repo = directory
        ),
        "Failed to rename branch to main"
      )
    }
  }

  local_branch <- gert::git_branch(repo = directory)
  push_refspec <- paste0("refs/heads/", local_branch)

  gertCall(
    gert::git_push(
      remote = "origin",
      refspec = push_refspec,
      set_upstream = TRUE,
      repo = directory,
      verbose = FALSE
    ),
    "Failed to push to GitHub"
  )
  
  return(invisible(TRUE))
}

test_that("validateRepoName accepts valid names", {
  # Valid names should not throw errors
  expect_silent(OmopStudyBuilder:::validateRepoName("my-study"))
  expect_silent(OmopStudyBuilder:::validateRepoName("MyStudy"))
  expect_silent(OmopStudyBuilder:::validateRepoName("my_study"))
  expect_silent(OmopStudyBuilder:::validateRepoName("my.study"))
  expect_silent(OmopStudyBuilder:::validateRepoName("study-123"))
  expect_silent(OmopStudyBuilder:::validateRepoName("Study_2024.v1"))
})

test_that("validateRepoName rejects invalid names", {
  # Spaces
  expect_error(
    OmopStudyBuilder:::validateRepoName("My Study"),
    "Repository name cannot contain spaces"
  )

  # Special characters
  expect_error(
    OmopStudyBuilder:::validateRepoName("study@2024"),
    "Repository name contains invalid characters"
  )

  expect_error(
    OmopStudyBuilder:::validateRepoName("study#test"),
    "invalid characters"
  )

  # Leading hyphen
  expect_error(
    OmopStudyBuilder:::validateRepoName("-study"),
    "cannot start or end with hyphen"
  )

  # Trailing hyphen
  expect_error(
    OmopStudyBuilder:::validateRepoName("study-"),
    "cannot start or end with hyphen"
  )

  # Too long (>30 chars)
  long_name <- paste0(rep("a", 31), collapse = "")
  expect_error(
    OmopStudyBuilder:::validateRepoName(long_name),
    "Repository name too long"
  )
})

test_that("linkGitHub fails on non-existent directory", {
  expect_error(
    linkGitHub(
      directory = "nonexistent_dir_12345",
      repository = "test-repo"
    ),
    "Directory does not exist"
  )
})

test_that("linkGitHub requires repository parameter", {
  temp_dir <- tempdir()
  expect_error(
    linkGitHub(directory = temp_dir),
    "missing, with no default"
  )
})

test_that("createStudyGitIgnore creates .gitignore file", {
  temp_dir <- file.path(tempdir(), "test_gitignore")
  dir.create(temp_dir, showWarnings = FALSE, recursive = TRUE)
  on.exit(unlink(temp_dir, recursive = TRUE), add = TRUE)

  OmopStudyBuilder:::createStudyGitIgnore(temp_dir)
  expect_true(file.exists(file.path(temp_dir, ".gitignore")))
})

test_that("createStudyGitIgnore has correct content", {
  temp_dir <- file.path(tempdir(), "test_gitignore_content")
  dir.create(temp_dir, showWarnings = FALSE, recursive = TRUE)
  on.exit(unlink(temp_dir, recursive = TRUE), add = TRUE)

  OmopStudyBuilder:::createStudyGitIgnore(temp_dir)
  content <- readLines(file.path(temp_dir, ".gitignore"))

  # Check for essential patterns
  expect_true(any(grepl("results/", content, fixed = TRUE)))
  expect_true(any(grepl(".Rproj.user", content, fixed = TRUE)))
  expect_true(any(grepl(".RData", content, fixed = TRUE)))
  expect_true(any(grepl("renv/library/", content, fixed = TRUE)))
  expect_true(any(grepl(".DS_Store", content, fixed = TRUE)))
  expect_true(any(grepl("Dockerfile", content, fixed = TRUE)))

  # Check for comments
  expect_true(any(grepl("# R artifacts", content, fixed = TRUE)))
  expect_true(any(grepl("# Study outputs", content, fixed = TRUE)))
})

test_that("createStudyGitIgnore merges with existing .gitignore", {
  temp_dir <- file.path(tempdir(), "test_gitignore_merge")
  dir.create(temp_dir, showWarnings = FALSE, recursive = TRUE)
  on.exit(unlink(temp_dir, recursive = TRUE), add = TRUE)

  # Create existing .gitignore with custom content
  existing_content <- c("custom_file.txt", "my_folder/", "*.log")
  writeLines(existing_content, file.path(temp_dir, ".gitignore"))

  # Add study .gitignore
  OmopStudyBuilder:::createStudyGitIgnore(temp_dir)

  # Read final content
  final_content <- readLines(file.path(temp_dir, ".gitignore"))

  # Check custom content preserved
  expect_true("custom_file.txt" %in% final_content)
  expect_true("my_folder/" %in% final_content)
  expect_true("*.log" %in% final_content)

  # Check study content added
  expect_true(any(grepl("results/", final_content, fixed = TRUE)))
  expect_true(any(grepl(".Rproj.user", final_content, fixed = TRUE)))

  # Check no duplicates created
  expect_equal(sum(final_content == "custom_file.txt"), 1)
})

test_that("createStudyGitIgnore handles duplicate entries gracefully", {
  temp_dir <- file.path(tempdir(), "test_gitignore_duplicates")
  dir.create(temp_dir, showWarnings = FALSE, recursive = TRUE)
  on.exit(unlink(temp_dir, recursive = TRUE), add = TRUE)

  # Create existing .gitignore with some study patterns already present
  existing_content <- c(".RData", "results/", "custom.txt")
  writeLines(existing_content, file.path(temp_dir, ".gitignore"))

  # Add study .gitignore
  OmopStudyBuilder:::createStudyGitIgnore(temp_dir)

  # Read final content
  final_content <- readLines(file.path(temp_dir, ".gitignore"))

  # Check no duplicates of existing entries
  expect_equal(sum(final_content == ".RData"), 1)
  expect_equal(sum(final_content == "results/"), 1)

  # Check custom entry preserved
  expect_true("custom.txt" %in% final_content)
})

test_that("createStudyGitIgnore trims whitespace before checking duplicates", {
  temp_dir <- file.path(tempdir(), "test_gitignore_trimws")
  dir.create(temp_dir, showWarnings = FALSE, recursive = TRUE)
  on.exit(unlink(temp_dir, recursive = TRUE), add = TRUE)

  existing_content <- c(".RData   ", "  results/**/*.csv", "custom.txt  ")
  writeLines(existing_content, file.path(temp_dir, ".gitignore"))

  OmopStudyBuilder:::createStudyGitIgnore(temp_dir)

  final_content <- readLines(file.path(temp_dir, ".gitignore"))

  expect_equal(sum(final_content == ".RData"), 1)
  expect_equal(sum(final_content == "results/**/*.csv"), 1)
  expect_true("custom.txt" %in% final_content)
})

test_that("ensureGitIdentity errors when git identity is missing and user info is absent", {
  skip_if_not_installed("gert")

  temp_dir <- tempfile("test_git_identity_missing-")
  temp_home <- tempfile("git-home-")
  old_home <- Sys.getenv("HOME", unset = NA_character_)
  old_userprofile <- Sys.getenv("USERPROFILE", unset = NA_character_)

  on.exit({
    unlink(temp_dir, recursive = TRUE)
    unlink(temp_home, recursive = TRUE)
    Sys.unsetenv(c("GIT_CONFIG_NOSYSTEM", "XDG_CONFIG_HOME", "HOME", "USERPROFILE"))

    if (!is.na(old_home)) {
      Sys.setenv(HOME = old_home)
    }

    if (!is.na(old_userprofile)) {
      Sys.setenv(USERPROFILE = old_userprofile)
    }
  }, add = TRUE)

  dir.create(temp_dir, showWarnings = FALSE, recursive = TRUE)
  dir.create(temp_home, showWarnings = FALSE, recursive = TRUE)
  Sys.setenv(
    GIT_CONFIG_NOSYSTEM = "1",
    XDG_CONFIG_HOME = temp_home,
    HOME = temp_home,
    USERPROFILE = temp_home
  )

  try(gert::git_config_global_set("user.name", NULL), silent = TRUE)
  try(gert::git_config_global_set("user.email", NULL), silent = TRUE)
  gert::git_init(temp_dir)

  expect_error(
    OmopStudyBuilder:::ensureGitIdentity(temp_dir, NULL),
    "Git identity not configured and no GitHub user information was provided"
  )
})

test_that("setupGitRemote pushes the actual local branch to a bare remote", {
  skip_if_not_installed("gert")

  repo_dir <- tempfile("test-linkGitHub-local-")
  remote_dir <- tempfile("test-linkGitHub-remote-")

  on.exit({
    unlink(repo_dir, recursive = TRUE)
    unlink(remote_dir, recursive = TRUE)
  }, add = TRUE)

  dir.create(repo_dir, recursive = TRUE)
  dir.create(remote_dir, recursive = TRUE)

  gert::git_init(repo_dir)
  gert::git_init(remote_dir, bare = TRUE)
  gert::git_config_set("user.name", "Test User", repo = repo_dir)
  gert::git_config_set("user.email", "test@example.com", repo = repo_dir)

  writeLines("hello", file.path(repo_dir, "README.md"))

  expect_no_error(
    OmopStudyBuilder:::setupGitRemote(
      directory = repo_dir,
      clone_url = remote_dir,
      user_info = list(login = "test", name = "Test User", email = "test@example.com"),
      created_repo = FALSE
    )
  )

  local_branch <- gert::git_branch(repo = repo_dir)
  remote_branches <- gert::git_branch_list(repo = remote_dir)
  expect_true(local_branch %in% remote_branches$name)
})

test_that("setupGitRemote uses main branch for fresh repos", {
  skip_if_not_installed("gert")

  repo_dir <- tempfile("test-linkGitHub-local-main-")
  remote_dir <- tempfile("test-linkGitHub-remote-main-")

  on.exit({
    unlink(repo_dir, recursive = TRUE)
    unlink(remote_dir, recursive = TRUE)
  }, add = TRUE)

  dir.create(repo_dir, recursive = TRUE)
  dir.create(remote_dir, recursive = TRUE)

  gert::git_init(repo_dir)
  gert::git_init(remote_dir, bare = TRUE)
  gert::git_config_set("user.name", "Test User", repo = repo_dir)
  gert::git_config_set("user.email", "test@example.com", repo = repo_dir)

  writeLines("hello", file.path(repo_dir, "README.md"))

  expect_no_error(
    OmopStudyBuilder:::setupGitRemote(
      directory = repo_dir,
      clone_url = remote_dir,
      user_info = list(login = "test", name = "Test User", email = "test@example.com"),
      created_repo = TRUE
    )
  )

  remote_branches <- gert::git_branch_list(repo = remote_dir)
  expect_true("main" %in% remote_branches$name)
})

test_that("linkGitHub validates directory parameter", {
  expect_error(
    linkGitHub(directory = NULL, repository = "test"),
    class = "error"
  )

  expect_error(
    linkGitHub(directory = "", repository = "test"),
    "Directory does not exist"
  )
})

test_that("linkGitHub validates repository parameter", {
  temp_dir <- tempdir()

  expect_error(
    linkGitHub(directory = temp_dir, repository = NULL),
    "can not be NULL"
  )

  expect_error(
    linkGitHub(directory = temp_dir, repository = ""),
    "must have at least 1 characters"
  )
})

# ============================================================================
# MANUAL/INTEGRATION TESTS
# These tests require real GitHub authentication and network access
# Run manually with: testthat::test_file("test-linkGitHub.R", filter = "manual")
# ============================================================================

test_that("linkGitHub full workflow (MANUAL TEST - requires GITHUB_PAT)", {
  skip_if_not(Sys.getenv("RUN_MANUAL_TESTS") == "true", 
              "Manual test - set RUN_MANUAL_TESTS=true to run (requires GITHUB_PAT)")
  skip_if_not(nzchar(Sys.getenv("GITHUB_PAT")), "GITHUB_PAT not set")
  skip_if_not_installed("gh")

  # WARNING: This creates a real GitHub repository!
  # You'll need to delete it manually after testing
  test_repo_name <- paste0("test-omop-study-", format(Sys.time(), "%Y%m%d-%H%M%S"))
  temp_dir <- file.path(tempdir(), test_repo_name)
  dir.create(temp_dir, showWarnings = FALSE, recursive = TRUE)
  on.exit(unlink(temp_dir, recursive = TRUE), add = TRUE)

  # Create a dummy file to commit
  writeLines("# Test Study", file.path(temp_dir, "README.md"))

  # Run linkGitHub
  result <- linkGitHub(
    directory = temp_dir,
    repository = test_repo_name,
    organisation = NULL,  # Creates under personal account
    private = TRUE
  )

  # Verify result
  expect_true(grepl("github.com", result))
  expect_true(grepl(test_repo_name, result))

  # Cleanup: Delete the test repository
  # NOTE: You may need to delete this manually from GitHub
  message("Test repository created: ", result)
  message("Please delete manually from GitHub if needed")
})

test_that("initStudy with GitHub integration (MANUAL TEST)", {
  skip_if_not(Sys.getenv("RUN_MANUAL_TESTS") == "true", 
              "Manual test - set RUN_MANUAL_TESTS=true to run (requires GITHUB_PAT)")
  skip_if_not(nzchar(Sys.getenv("GITHUB_PAT")), "GITHUB_PAT not set")
  skip_if_not_installed("gh")

  test_repo_name <- paste0("test-init-study-", format(Sys.time(), "%Y%m%d-%H%M%S"))
  temp_dir <- file.path(tempdir(), test_repo_name)
  on.exit(unlink(temp_dir, recursive = TRUE), add = TRUE)

  # Create study with GitHub integration
  initStudy(
    directory = temp_dir,
    studyTitle = "Test Study",
    studyLeads = "Test Lead",
    repository = test_repo_name,
    organisation = NULL,
    private = TRUE
  )

  # Verify directory structure
  expect_true(dir.exists(temp_dir))
  expect_true(file.exists(file.path(temp_dir, "README.md")))
  expect_true(dir.exists(file.path(temp_dir, ".git")))
  expect_true(file.exists(file.path(temp_dir, ".gitignore")))

  message("Test repository created: https://github.com/YOUR_USERNAME/", test_repo_name)
  message("Please delete manually from GitHub")
})

---
title: "reviewStudy"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{reviewStudy}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(OmopStudyBuilder)
```

## Overview

When you receive a study package (or before you ship one), it is useful to quickly inspect:

- What files are included (R scripts, JSON, CSV, Excel)
- Whether `renv.lock` exists, and what it contains
- Whether any dependencies look surprising for the study type

`OmopStudyBuilder` provides two helpers:

- `reviewStudyCode()` summarises the contents of a directory
- `reviewStudyDependencies()` summarises dependencies based on `renv.lock`

## Review files in a study folder

Point `reviewStudyCode()` at the folder you want to inspect (often `studyCode/` or `diagnosticsCode/`).

The simplest pattern is to run these commands from within the project you want to review (for example, open `studyCode.Rproj` or `diagnosticsCode.Rproj`).

```{r reviewStudyCode-example, eval=FALSE}
reviewStudyCode(".")
```

## Review dependencies

`reviewStudyDependencies()` expects a `renv.lock` file in the directory.
Use `type = "analysis"` for analysis code, or `type = "reporting"` for reporting code.

```{r reviewStudyDependencies-example, eval=FALSE}
reviewStudyDependencies(
  dir = ".",
  type = "analysis"
)
```

# Instructions for Running This Study

This document provides detailed instructions for running the different components of this study.

---

{{#HAS_DIAGNOSTICS}}
## Running the Diagnostics Code

The diagnostics code is located in the `diagnosticsCode` folder and should be run before the main study analysis.

### Steps

1)  Download this entire repository (you can download as a zip folder using Code → Download ZIP, or you can use GitHub Desktop).
2)  Open the project `diagnosticsCode.Rproj` from the `diagnosticsCode` directory in RStudio (when inside the project, you will see its name on the top-right of your RStudio session)
3)  Open the `codeToRun.R` file - this is the only file you should need to interact with.
4)  Install the required packages using `renv::restore()` and then load these libraries
5)  Add your database specific parameters (name of database, schema name with OMOP data, schema name to write results, table name stem for results to be saved in the result schema).
6)  Create a cdm using CDMConnector (see <https://darwin-eu.github.io/CDMConnector/articles/a04_DBI_connection_examples.html> for connection examples for different dbms). Achilles tables must be included in your cdm reference.
7)  Run `source(here("runStudy.R"))` to run the diagnostics.

---

{{/HAS_DIAGNOSTICS}}
{{#HAS_STUDY}}
## Running the Study Code

The main study analysis code is located in the `studyCode` folder.

### Steps

1)  Download this entire repository (you can download as a zip folder using Code → Download ZIP, or you can use GitHub Desktop).
2)  Open the project `studyCode.Rproj` from the `studyCode` directory in RStudio (when inside the project, you will see its name on the top-right of your RStudio session)
3)  Open the `codeToRun.R` file - this is the only file you should need to interact with.
4)  Install the required packages using `renv::restore()` and then load these libraries
5)  Add your database specific parameters (name of database, schema name with OMOP data, schema name to write results, table name stem for results to be saved in the result schema).
6)  Create a cdm using CDMConnector (see <https://darwin-eu.github.io/CDMConnector/articles/a04_DBI_connection_examples.html> for connection examples for different dbms).
7)  Run `source(here("runStudy.R"))` to run the analysis.

---

{{/HAS_STUDY}}
{{#HAS_DIAGNOSTICS}}
## Running the Diagnostics Shiny App

The diagnostics shiny app is located in the `diagnosticsShiny` folder and is used to explore diagnostic outputs.

### Steps

1)  Ensure your diagnostic results are placed in the `results` folder
2)  Open the project in RStudio
3)  Restore the R environment using `renv::restore()`
4)  Run the app using `shiny::runApp()`

---

{{/HAS_DIAGNOSTICS}}
{{#HAS_STUDY}}
## Running the Study Shiny App

The study shiny app is located in the `studyShiny` folder and is used to explore study results.

### Steps

1)  Ensure your study results are placed in the `results` folder
2)  Open the project in RStudio
3)  Restore the R environment using `renv::restore()`
4)  Run the app using `shiny::runApp()`

---

{{/HAS_STUDY}}

## Additional Resources

- [CDMConnector Database Connection Examples](https://darwin-eu.github.io/CDMConnector/articles/a04_DBI_connection_examples.html)
- [OmopStudyBuilder Documentation](https://oxford-pharmacoepi.github.io/OmopStudyBuilder/)

---

*This study was generated using [OmopStudyBuilder](https://oxford-pharmacoepi.github.io/OmopStudyBuilder/).*

# {{STUDY_TITLE}}
<img src="https://img.shields.io/badge/Study%20Status-{{STUDY_STATUS}}-blue.svg" alt="Study Status: {{STUDY_STATUS}}">

- **Study title**: {{STUDY_TITLE}}
- **Study start date**: {{STUDY_START}}
- **Study leads**: {{STUDY_LEADS}}
- **Study end date**: {{STUDY_END}}
- **Publications**: {{PUBLICATIONS}}

## Study Description

{{STUDY_DESCRIPTION}}

---

## Repository Organization

This repository is organised as follows:

{{FOLDER_STRUCTURE}}

---

For detailed instructions on how to run the study, please see the [INSTRUCTIONS.md](./INSTRUCTIONS.md) file.

---

*This study template was generated using [OmopStudyBuilder](https://oxford-pharmacoepi.github.io/OmopStudyBuilder/).*

---
title: "initStudy"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{initStudy}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup, include=FALSE}
library(OmopStudyBuilder)
```

## Introduction

`initStudy()` is the starting point for building a study project with the
**OMOP Common Data Model (CDM)** using the `OmopStudyBuilder` package.

It generates a fully structured project folder containing:

* Templates for study diagnostics
* Templates for analysis code
* Shiny apps for diagnostics and reporting
* A project README
* A reproducible, opinionated layout aligned with [OxInfer](https://oxford-pharmacoepi.github.io/Oxinfer/onboarding/code_review.html) conventions

This allows you to immediately focus on writing study-specific code instead of
manually assembling the surrounding infrastructure.

In this vignette you will learn how to:

* Create a new study project using `initStudy()`
* Explore the folder structure that is generated
* Control whether diagnostics and/or study analysis templates are included
* Understand next steps after project creation

---

## Basic usage

The simplest way to use `initStudy()` is to provide a directory path.
If the directory:

* **Does not exist**: it will be created
* **Exists but is empty**: it will be used
* **Exists and contains files**: an error will be thrown

To avoid modifying your real project directories, this vignette uses a temporary
folder.

```{r basic-usage, message=FALSE}
# Create a temporary root for this vignette
study_root <- file.path(tempdir(), "SampleStudy")

initStudy(study_root)

# Show top-level contents created by initStudy()
list.files(study_root)
```

You should see something like:

* `diagnosticsCode/` – diagnostic scripts and helpers
* `diagnosticsShiny/` – skeleton diagnostics Shiny app
* `studyCode/` – scripts for running the main analyses
* `studyShiny/` – skeleton reporting Shiny app
* `README.md` – overview of the study project

To inspect the full structure:

```{r list-recursive}
list.files(study_root, recursive = TRUE)
```

---

## Diagnostics-only and study-only setups

You can control which parts of the template are created with the
`diagnostics` and `study` arguments.

### Diagnostics-only project

```{r diagnostics-only, message=FALSE}
diag_root <- file.path(tempdir(), "DiagnosticsOnly")

initStudy(
  directory   = diag_root,
  diagnostics = TRUE,
  study       = FALSE
)

list.files(diag_root)
```

### Study-only project (no diagnostics templates)

```{r study-only, message=FALSE}
study_only_root <- file.path(tempdir(), "StudyOnly")

initStudy(
  directory   = study_only_root,
  diagnostics = FALSE,
  study       = TRUE
)

list.files(study_only_root)
```

---

## What to do after `initStudy()`

After generating the project structure, you will mainly work with these folders and files:

- `studyCode/`  
  - `studyCode.Rproj`: open this in RStudio to work on the main analysis code. It sets the working directory so all relative paths behave correctly.  
  - `codeToRun.R`: where you add database connection details, schemas, and global settings
  - `cohorts/instantiateCohorts.R`: defines the study cohorts using your codelists and cohort-building functions. You customise this to match your phenotype definitions.  
  - `runStudy.R`: orchestrates the analysis steps (cohort creation, summaries, analyses, export). 

- `diagnosticsCode/`  
  - `diagnosticsCode.Rproj`: open this project to work on diagnostics. It keeps diagnostics code separate from the main analysis project.  
  - `codeToRun.R`: similar to the study version, but dedicated to diagnostics. You set connection details and schemas used for running phenotype diagnostics.  
  - `cohorts/instantiateCohorts.R`: defines cohorts for diagnostic checking, often mirroring or simplifying the main study cohorts.  
  - `runStudy.R`: runs the PhenotypeDiagnostics workflow and exports diagnostic results. You might tweak options, but the overall flow is usually left intact.

- `studyShiny/`  
  - For a shiny app to explore study results. You extend this with plots, tables, and filters built on top of your exported outputs.

- `diagnosticsShiny/`  
  - Contains a minimal Shiny app for exploring diagnostic outputs. Use it to review cohort characteristics and diagnostic results interactively across data partners.

---

## Building and Distributing Your Study

Once your study code is ready, `OmopStudyBuilder` provides functions to package it into a Docker image and distribute it to data partners.

### Building the Docker Image

```{r build-docker, eval=FALSE}
# Build a Docker image containing your study code
dockeriseStudy(
  image_name = "omop-study-study-code",
  path = file.path(study_root, "studyCode")
)
#> Building Docker image: omop-study-study-code
#> This may take some minutes on first build...
#> ✔ Image built successfully: omop-study-study-code
```

The image contains:
- Your exact R version
- All package dependencies (from renv.lock)
- Your study code
- Database drivers and system dependencies

### Distribution Options

#### Option 1: Docker Hub (Online)

Push to Docker Hub for easy sharing with connected partners:

```{r push-image, eval=FALSE}
pushDockerImage(
  image_name = "omop-study-study-code",
  repo = "yourusername/myomopstudy"
)
#> ✔ Image pushed to Docker Hub!
```

Data partners can then pull and run the image.

They can then run either an interactive RStudio Server session, or run the study in automated mode.

```{r pull-run, eval=FALSE}
library(OmopStudyBuilder)

# Interactive (requires image built with dockeriseStudy(useRStudio = TRUE)):
runRStudio(
  image_name = "yourusername/myomopstudy:latest",
  results_path = "./results"
)
```

### Interactive vs Automated Execution

**RStudio Server** runs the Docker image and opens a browser to the RStudio Server URL:

```{r rstudio-run, eval=FALSE}
# Requires image built with dockeriseStudy(useRStudio = TRUE)
runRStudio(
  image_name = "omop-study-study-code",
  results_path = "./results"
)
```

Partners edit credentials in `codeToRun.R` and run the study interactively.

**Automated execution** (for programmatic workflows):

```{r auto-run, eval=FALSE}
runStudy(
  image_name = "omop-study-study-code",
  results_path = "./results",
  data_path = "path/to/data"
)
```

To stop any running OmopStudyBuilder containers, use:

```{r stop-containers, eval=FALSE}
stopStudy(image_name = "omop-study-study-code")
```

For more operational details, see the README files generated into your study folders (for example `studyCode/README.md` and `diagnosticsCode/README.md`).

---

---
title: "dockerWrapper"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{dockerWrapper}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(OmopStudyBuilder)
```

## Overview

This vignette describes the Docker helper functions in `OmopStudyBuilder`:

- `dockeriseStudy()` builds a Docker image for your study
- `runRStudio()` starts an interactive RStudio Server session inside the image
- `runStudy()` runs your study script in automated mode (streaming logs)
- `stopStudy()` stops containers started by `runRStudio()`/`runStudy()`
- `pushDockerImage()` tags and pushes your image to Docker Hub

All Docker calls require Docker to be installed and the Docker daemon to be running.

## Build an image

`dockeriseStudy()` expects an `renv.lock` (and renv activation files) in the directory you point it at.
By default it will also snapshot your dependencies before building.

```{r dockeriseStudy-example, eval=FALSE}
# From the studyCode folder (or pass path=...)
dockeriseStudy(
  image_name = "my-study",
  path = "./studyCode",
  useRStudio = FALSE
)
```

If you want `runRStudio()`, build with an RStudio base image:

```{r dockeriseStudy-rstudio, eval=FALSE}
dockeriseStudy(
  image_name = "my-study",
  path = "./studyCode",
  useRStudio = TRUE
)
```

If your `renv.lock` includes GitHub packages and you hit GitHub rate limits during the build,
pass a token:

```{r dockeriseStudy-github-token, eval=FALSE}
dockeriseStudy(
  path = "./studyCode",
  github_token = Sys.getenv("GITHUB_PAT")
)
```

## Run interactively (RStudio Server)

`runRStudio()` runs the Docker image and opens a browser to the RStudio Server URL.
Results are written to `results_path` on your host.

```{r runRStudio-example, eval=FALSE}
runRStudio(
  image_name = "my-study",
  results_path = "./results"
)
```

To pass runtime settings to the container, create a `.env` file in your working directory
and it will be used automatically (or provide `env_file = "path/to/.env"`).

## Run in automated mode

`runStudy()` executes an R script in the container (default `codeToRun.R`) and streams
stdout/stderr to your console.

```{r runStudy-example, eval=FALSE}
runStudy(
  image_name = "my-study",
  results_path = "./results",
  data_path = "/path/to/cdm/data",
  script_path = "codeToRun.R"
)
```

## Stop containers

Use `stopStudy()` to stop containers started by `runRStudio()` and/or `runStudy()`.

```{r stopStudy-example, eval=FALSE}
stopStudy(image_name = "my-study")
```

## Push to Docker Hub

`pushDockerImage()` will tag `image_name:latest` and push it to a Docker Hub repository.

```{r pushDockerImage-example, eval=FALSE}
pushDockerImage(
  image_name = "my-study",
  repo = "yourusername/my-study"
)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{generateContainerName}
\alias{generateContainerName}
\title{Generate unique Docker container name}
\usage{
generateContainerName(base_name, suffix = NULL, max_tries = 100)
}
\arguments{
\item{base_name}{Base name for container}

\item{suffix}{Optional suffix (e.g., "-run")}

\item{max_tries}{Maximum attempts to find unique name}
}
\value{
Unique container name
}
\description{
Generate unique Docker container name
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reviewStudy.R
\name{reviewStudy}
\alias{reviewStudy}
\title{Review a study directory}
\usage{
reviewStudy(dir, code = TRUE, dependencies = TRUE, type = "analysis")
}
\arguments{
\item{dir}{Path to the study directory.}

\item{code}{If `TRUE`, summarises R, JSON, CSV, and Excel files found in
the directory.}

\item{dependencies}{If `TRUE`, summarises the renv.lock dependencies.}

\item{type}{Whether the R project is for analysis code or study reporting.
Only used when `dependencies = TRUE`.}
}
\value{
Invisibly returns `NULL`. Called for its side effects of printing
  summaries to the console.
}
\description{
Summarises the code and/or renv dependencies in a study directory.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OmopStudyBuilder-package.R
\docType{package}
\name{OmopStudyBuilder-package}
\alias{OmopStudyBuilder}
\alias{OmopStudyBuilder-package}
\title{OmopStudyBuilder: Build Reproducible Network Studies for OMOP Common Data Model}
\description{
Streamlines the setup and execution of network studies using the Observational Medical Outcomes Partnership (OMOP) Common Data Model (CDM). Creates standardised project structures with template code, manages dependencies with 'renv', provides code review utilities, and supports containerised execution with 'Docker' for reproducible multi-site studies. Includes 'GitHub' integration for collaboration and version control.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://oxford-pharmacoepi.github.io/OmopStudyBuilder/}
  \item Report bugs at \url{https://github.com/oxford-pharmacoepi/OmopStudyBuilder/issues}
}

}
\author{
\strong{Maintainer}: Folu Akintola \email{folu.akintola@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-0588-2298}{ORCID})

Authors:
\itemize{
  \item Edward Burn \email{edward.burn@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-9286-1128}{ORCID})
  \item Martí Català \email{marti.catalasabate@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0003-3308-9905}{ORCID})
}

Other contributors:
\itemize{
  \item Marta Alcalde-Herraiz \email{marta.alcaldeherraiz@ndorms.ox.ac.uk} (\href{https://orcid.org/0009-0002-4405-1814}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{buildDockerLabels}
\alias{buildDockerLabels}
\title{Build Docker labels for OmopStudyBuilder containers}
\usage{
buildDockerLabels(image_name, mode)
}
\arguments{
\item{image_name}{Docker image name}

\item{mode}{Container mode ("rstudio" or "run")}
}
\value{
Character vector of Docker label arguments
}
\description{
Build Docker labels for OmopStudyBuilder containers
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{setupGitRemote}
\alias{setupGitRemote}
\title{Setup git remote and push}
\usage{
setupGitRemote(directory, clone_url, user_info = NULL, created_repo = FALSE)
}
\description{
Setup git remote and push
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{ensureDocker}
\alias{ensureDocker}
\title{Check if Docker daemon is running}
\usage{
ensureDocker()
}
\value{
TRUE if Docker is available, throws error otherwise
}
\description{
Check if Docker daemon is running
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{checkRepoAvailable}
\alias{checkRepoAvailable}
\title{Check if repository name is available}
\usage{
checkRepoAvailable(owner, repo)
}
\description{
Check if repository name is available
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/initStudy.R
\name{initStudy}
\alias{initStudy}
\title{Creates initial directory for an OMOP CDM network study}
\usage{
initStudy(
  directory,
  diagnostics = TRUE,
  study = TRUE,
  studyTitle = NULL,
  studyLeads = NULL,
  studyDescription = NULL,
  repository = NULL,
  organisation = NULL,
  private = TRUE
)
}
\arguments{
\item{directory}{Path to a directory that will be used as the root folder
for the study. If it does not exist, it will be created. The directory
must be empty if it already exists.}

\item{diagnostics}{A single TRUE or FALSE value.
If TRUE (the default), the function creates the `diagnosticsCode/`
and `diagnosticsShiny/` folders using the package templates.
If FALSE, these diagnostics folders are not created.}

\item{study}{A single TRUE or FALSE value.
If TRUE (the default), the function creates the `studyCode/`
and `studyShiny/` folders using the package templates.
If FALSE, these study folders are not created.}

\item{studyTitle}{Character string with the study title. If NULL (default),
uses the directory basename.}

\item{studyLeads}{Character string with study leads. If NULL (default),
leaves a placeholder.}

\item{studyDescription}{Character string with study description. If NULL (default),
leaves a placeholder.}

\item{repository}{Optional GitHub repository name. If provided, creates a GitHub
repository and links it to the study. Requires the \code{gh} and \code{gert}
packages and GitHub authentication (for example via \code{GITHUB_PAT}).}

\item{organisation}{Optional GitHub organisation name. If NULL (default), creates
repository under your personal account. Only used when \code{repository} is provided.}

\item{private}{Logical. If TRUE (default), creates a private GitHub repository.
Only used when \code{repository} is provided.}
}
\value{
Project directory will be created
}
\description{
Creates initial directory for an OMOP CDM network study
}
\examples{
# Create a study called "SampleStudy" in a temporary directory
study_root <- file.path(tempdir(), "SampleStudy")
initStudy(study_root)

# Inspect the top-level contents
list.files(study_root)

# Create another study with custom metadata
study_root2 <- file.path(tempdir(), "DiabetesStudy")
initStudy(study_root2,
          studyTitle = "Diabetes Prevalence Study",
          studyLeads = "Dr. Smith, Dr. Jones")

\dontrun{
# Create study with GitHub integration (requires GITHUB_PAT)
# Set PAT for current session:
Sys.setenv(GITHUB_PAT = "your_token_here")

study_root3 <- file.path(tempdir(), "GitHubStudy")
initStudy(
  directory = study_root3,
  repository = "my-omop-study",
  organisation = "oxford-pharmacoepi",
  private = TRUE
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{checkGitHubAuth}
\alias{checkGitHubAuth}
\title{Check GitHub authentication}
\usage{
checkGitHubAuth()
}
\description{
Check GitHub authentication
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{dockerExec}
\alias{dockerExec}
\title{Execute a Docker command}
\usage{
dockerExec(args, error_message = "Docker command failed")
}
\arguments{
\item{args}{Character vector of Docker command arguments}

\item{error_message}{Custom error message if command fails}
}
\value{
Command output as character vector (returned invisibly to avoid console clutter)
}
\description{
Execute a Docker command
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{dockerQuery}
\alias{dockerQuery}
\title{Run a Docker query command and return cleaned output}
\usage{
dockerQuery(args)
}
\arguments{
\item{args}{Character vector of Docker command arguments}
}
\value{
Output lines, or empty if the command fails
}
\description{
Run a Docker query command and return cleaned output
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{stopStudy}
\alias{stopStudy}
\title{Stop a running OmopStudyBuilder container (automated or RStudio)}
\usage{
stopStudy(
  container = NULL,
  image_name = NULL,
  mode = c("any", "rstudio", "run"),
  all = FALSE
)
}
\arguments{
\item{container}{Optional container name or ID to stop directly.}

\item{image_name}{Optional Docker image name. Defaults to current directory name.}

\item{mode}{Which container(s) to stop: "any" (default), "rstudio", or "run".}

\item{all}{If TRUE, stops all running containers started by OmopStudyBuilder.}
}
\value{
TRUE if at least one container was stopped (invisibly)
}
\description{
Stops containers started by \code{runStudy()} and/or \code{runRStudio()}.
By default it stops containers for the image inferred from the current
directory name.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{processEnvFile}
\alias{processEnvFile}
\title{Process environment file argument}
\usage{
processEnvFile(env_file, default_env_file = ".env")
}
\arguments{
\item{env_file}{Path to .env file or NULL}

\item{default_env_file}{Default env file path to use when env_file is NULL}
}
\value{
Character vector of Docker env-file arguments (empty if no file)
}
\description{
Process environment file argument
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{autoDetectImageName}
\alias{autoDetectImageName}
\title{Auto-detect image name from current directory}
\usage{
autoDetectImageName(image_name = NULL)
}
\arguments{
\item{image_name}{Existing image name (if provided)}
}
\value{
Image name (lowercased directory basename)
}
\description{
Auto-detect image name from current directory
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{runRStudio}
\alias{runRStudio}
\title{Run RStudio Server for interactive study execution}
\usage{
runRStudio(
  image_name = NULL,
  results_path = "./results",
  env_file = NULL,
  port = 8787,
  password = NULL
)
}
\arguments{
\item{image_name}{Name of Docker image to run (default: auto-detected from directory)}

\item{results_path}{Path to save results (default: "./results")}

\item{env_file}{Optional path to a .env file (passed to Docker via --env-file).
If NULL and a `.env` file exists in the current working directory, it will be
used automatically.}

\item{port}{Port for RStudio Server (default: 8787, auto-finds next available if busy)}

\item{password}{RStudio password (default: auto-generated and displayed)}
}
\value{
Container ID (invisibly)
}
\description{
Note: The Docker image must include RStudio Server (i.e., be built with
`dockeriseStudy(useRStudio = TRUE)` which uses a `rocker/rstudio` base image).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{gitCommit}
\alias{gitCommit}
\title{Create git commit}
\usage{
gitCommit(
  directory,
  message,
  allow_nothing_to_commit = FALSE,
  error_message = "Failed to commit"
)
}
\description{
Create git commit
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{initializeGitRepo}
\alias{initializeGitRepo}
\title{Initialize local git repository}
\usage{
initializeGitRepo(directory)
}
\description{
Initialize local git repository
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{verifyImageExists}
\alias{verifyImageExists}
\title{Verify Docker image exists and provide helpful error}
\usage{
verifyImageExists(image_name, check_rstudio = FALSE)
}
\arguments{
\item{image_name}{Name of Docker image}

\item{check_rstudio}{If TRUE, also verify RStudio Server is present}
}
\value{
TRUE (invisibly) if checks pass, otherwise throws error
}
\description{
Verify Docker image exists and provide helpful error
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{createGitHubRepo}
\alias{createGitHubRepo}
\title{Create GitHub repository via API}
\usage{
createGitHubRepo(repository, organisation, private, description)
}
\description{
Create GitHub repository via API
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{createStudyGitIgnore}
\alias{createStudyGitIgnore}
\title{Create .gitignore for OMOP studies}
\usage{
createStudyGitIgnore(directory)
}
\description{
Create .gitignore for OMOP studies
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{ensureGitIdentity}
\alias{ensureGitIdentity}
\title{Ensure Git identity is configured}
\usage{
ensureGitIdentity(directory, user_info = NULL)
}
\description{
Checks if Git user.name and user.email are configured for commits.
If not configured, automatically sets them using GitHub account info
provided by the caller. This allows seamless Git operations without
requiring manual `git config` setup.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{runStudy}
\alias{runStudy}
\title{Run study in automated mode with real-time log streaming}
\usage{
runStudy(
  image_name = NULL,
  results_path = "./results",
  env_file = NULL,
  data_path = NULL,
  script_path = "codeToRun.R"
)
}
\arguments{
\item{image_name}{Name of Docker image to run (default: auto-detected from directory)}

\item{results_path}{Path to save results (default: "./results")}

\item{env_file}{Optional path to a .env file (passed to Docker via --env-file).
If NULL and a `.env` file exists in the current working directory, it will be
used automatically.}

\item{data_path}{Optional path to data directory (mounted at /data)}

\item{script_path}{Path to R script to execute (default: "codeToRun.R")}
}
\value{
Exit status (0 = success, non-zero = failure)
}
\description{
Run study in automated mode with real-time log streaming
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{pushDockerImage}
\alias{pushDockerImage}
\title{Push a Docker image to Docker Hub}
\usage{
pushDockerImage(
  image_name = NULL,
  repo,
  tag = "latest",
  username = NULL,
  password = NULL,
  logout = TRUE
)
}
\arguments{
\item{image_name}{Name of Docker image to push (default: auto-detected from directory)}

\item{repo}{Docker Hub repository (e.g., "username/repo" or "repo")}

\item{tag}{Tag to push (default: "latest")}

\item{username}{Docker Hub username (prompted if NULL)}

\item{password}{Docker Hub password or token (prompted if NULL, hidden input)}

\item{logout}{If TRUE, logs out after pushing}
}
\value{
Pushed image reference (invisibly)
}
\description{
Push a Docker image to Docker Hub
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{validateRepoName}
\alias{validateRepoName}
\title{Validate repository name for GitHub}
\usage{
validateRepoName(name)
}
\arguments{
\item{name}{Repository name to validate}
}
\value{
TRUE if valid, throws error otherwise
}
\description{
Checks if a repository name follows GitHub naming rules.
Throws an error with helpful suggestions if invalid.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linkGitHub.R
\name{linkGitHub}
\alias{linkGitHub}
\title{Link study directory to GitHub repository}
\usage{
linkGitHub(
  directory,
  repository,
  organisation = NULL,
  private = TRUE,
  description = NULL
)
}
\arguments{
\item{directory}{Path to study directory}

\item{repository}{GitHub repository name (will be sanitised if needed)}

\item{organisation}{GitHub organisation name. If NULL (default), creates repo
under your personal account}

\item{private}{Logical. If TRUE (default), creates a private repository}

\item{description}{Repository description. If NULL, auto-generated from directory name}
}
\value{
GitHub repository URL (invisibly)
}
\description{
Creates a new GitHub repository and connects it to an existing study directory.
Initializes git, creates .gitignore, commits all files, and pushes to GitHub.
}
\section{Requirements}{

\itemize{
  \item \strong{GitHub authentication}: Set up via GITHUB_PAT, gh CLI, or git credentials
  \item \strong{R package 'gh'}: Install with \code{install.packages("gh")}
  \item \strong{R package 'gert'}: Install with \code{install.packages("gert")}
}
}

\section{Authentication}{

This function needs credentials for both:
\itemize{
  \item \strong{GitHub API access}: used by \code{gh} to check your account and create the repository
  \item \strong{Git transport authentication}: used by \code{gert} to push the local repository to GitHub
}

Recommended setup:
\itemize{
  \item \strong{GITHUB_PAT environment variable}: recommended for HTTPS authentication and works for both GitHub API calls and Git pushes to GitHub
  \item \strong{Stored Git credentials}: credentials in your OS credential store or Git credential helper can also work for both the API and push steps
  \item \strong{gh CLI}: may help set up GitHub authentication, but you may still need Git credentials available for the final push
  \item \strong{SSH}: supported when your remote/auth setup is configured accordingly
}

To create a Personal Access Token (PAT):
\enumerate{
  \item Visit \url{https://github.com/settings/tokens}
  \item Generate a token with \code{repo} scope
  \item Set for current session: \code{Sys.setenv(GITHUB_PAT = "your_token_here")}
  \item Or add to .Renviron: \code{GITHUB_PAT='your_token_here'} and restart R
}
}

\examples{
\dontrun{
library(OmopStudyBuilder)
library(here)

# Authenticate (choose one method):
# 1. Set GITHUB_PAT for current session
Sys.setenv(GITHUB_PAT = "your_token_here")
# 2. Or use gh CLI: gh auth login

# Create repo under personal account
linkGitHub(
  directory = here::here(),
  repository = "my-omop-study"
)

# Create repo under organisation
linkGitHub(
  directory = here::here(),
  repository = "diabetes-study",
  organisation = "oxford-pharmacoepi",
  private = TRUE
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{findAvailablePort}
\alias{findAvailablePort}
\title{Find next available port starting from a given port}
\usage{
findAvailablePort(start_port = 8787, max_tries = 10)
}
\arguments{
\item{start_port}{Starting port number to check}

\item{max_tries}{Maximum number of ports to try}
}
\value{
Available port number
}
\description{
Find next available port starting from a given port
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dockerWrapper.R
\name{dockeriseStudy}
\alias{dockeriseStudy}
\title{Build a Docker image for an OMOP study}
\usage{
dockeriseStudy(
  image_name = NULL,
  path = ".",
  useRStudio = FALSE,
  r_version = NULL,
  snapshot = TRUE,
  github_token = NULL
)
}
\arguments{
\item{image_name}{Name for the Docker image (default: auto-detected from directory)}

\item{path}{Path to study directory (default: current directory)}

\item{useRStudio}{Use RStudio Server base (TRUE) or r-ver base (FALSE, default)}

\item{r_version}{R version override (default: auto-detected from renv.lock)}

\item{snapshot}{Update renv.lock before building (default: TRUE)}

\item{github_token}{Optional GitHub token for installing GitHub packages during build}
}
\value{
Image name (invisibly - already printed to console)
}
\description{
Build a Docker image for an OMOP study
}
